        subroutine diagr( xorg, yorg, xsize, ysize, xmin, xmax, ymin, 
     >    ymax, xinc, yinc, xtext, ytext, head, fracol, ovrcol, x, y, 
     >    n, dshlen )
C ***
C ******************************************************************************
C ***
C ***   $Id: diagr.f,v 1.16 2002/12/28 15:44:05 michael Exp $
C ***
C ***                             19-NOV-1990 20:50:42.36
C ***
C ***   Zeichnet Diagramm: Kurve aus Array x,y mit Rahmen.
C ***
C ***   xorg, yorg    : Koordinaten in cm der linken unteren Ecke des
C ***                   Kastens, relativ zum aktuellen Ursprung der Zeichnung
C ***   xsize, ysize  : Laenge der x- bzw. y-Achse in cm.
C ***   xmin, xmax    : Kleinster und groesster x-Wert in dargest. Einheiten,
C ***                   xmin = xmax: automatisch
C ***   ymin, ymax    : Kleinster und groesster y-Wert in dargest. Einheiten
C ***                   ymin = ymax: automatisch
C ***   xinc, yinc    : Abstand zwischen zwei Achsenmarkierungen. 0: autom.
C ***   xtext, ytext  : Beschriftung der Achsen.
C ***   head          : Ueberschrift.
C ***   fracol        : Rahmenfarbe
C ***   ovrcol        : Kurvenfarbe
C ***   x, y          : Zu zeichnende Funktion.
C ***   n             : Anzahl Punkte [ x(1), .. x(n), y(1), .. y(n) ]
C ***   dshlen        : Gestrichelte Kurve: Strichlaenge, 0: kontinuierlich
C ***
C ***   Modifications:
C ***
C ***   $Log: diagr.f,v $
C ***   Revision 1.16  2002/12/28 15:44:05  michael
C ***   Fix of optimization for inverted axes.
C ***   Don't optimize away two point range.
C ***
C ***   Revision 1.15  2002/12/12 21:22:56  michael
C ***   Vertical lines were optimized away....
C ***
C ***   Revision 1.14  2002/12/11 18:51:25  michael
C ***   Small fix for ascending data.  End point was not detected correctly.
C ***
C ***
C ***
C ******************************************************************************
C ***

        real x(n), y(n)
        character*(*) xtext, ytext, head
        real l
        integer fracol, ovrcol
        logical xlab, ylab
        common /pltpar/ xscal, yscal, x0, y0, xc0, yc0, xc1, yc1, 
     >    xlab, ylab, xsiz, ysiz, hsiz, symsiz


c       xxmin = xmin
c       xxmax = xmax
c       yymin = ymin
c       yymax = ymax
        if( xmin .eq. xmax ) then
           xmin = 1.e37
           xmax = -1.e37
           do 1  i = 1, n
              xmin = min( xmin, x(i) )
              xmax = max( xmax, x(i) )
1          continue
           d = 0.1*(xmax - xmin)
           xmin = xmin - d
           xmax = xmax + d
           if( xmin .eq. xmax ) then    ! Konstante Funktion
              xmin = xmin - 0.1*xmin
              xmax = xmax + 0.1*xmin
           end if
        end if

        if( ymin .eq. ymax ) then
           ymin = 1.e37
           ymax = -1.e37
           do 2  i = 1, n
              ymin = min( ymin, y(i) )
              ymax = max( ymax, y(i) )
2          continue
           d = 0.1*(ymax - ymin)
           ymin = ymin - d
           ymax = ymax + d
           if( ymin .eq. ymax ) then    ! Konstante Funktion
              ymin = ymin - 0.1*ymin
              ymax = ymax + 0.1*ymin
           end if
        end if

        call clip( xorg, yorg, xsize, ysize, xmin, xmax, ymin, ymax )

        call newpen( ovrcol )
        call over( x, y, n, dshlen )

        call frame( xorg, yorg, xsize, ysize, xmin, xmax, ymin, 
     >    ymax, xinc, yinc, xtext, ytext, head, fracol )

        return
        end
        subroutine frame( xorg, yorg, xsize, ysize, xmin, xmax, ymin, 
     >    ymax, xinc, yinc, xtext, ytext, head, fracol )
C ***
C ******************************************************************************
C ***
C ***                            16-MAR-1990 13:18:14.64
C ***
C ***   Zeichnet Rahmen und setzt Cliprahmen
C ***
C ***   xorg, yorg    : Koordinaten in cm der linken unteren Ecke des
C ***                   Kastens, relativ zum aktuellen Ursprung der Zeichnung
C ***   xsize, ysize  : Laenge der x- bzw. y-Achse in cm.
C ***   xmin, xmax    : Kleinster und groesster x-Wert in dargest. Einheiten
C ***   ymin, ymax    : Kleinster und groesster y-Wert in dargest. Einheiten
C ***   xinc, yinc    : Abstand zwischen zwei Achsenmarkierungen. 0: autom.
C ***   xtext, ytext  : Beschriftung der Achsen.
C ***   head          : Ueberschrift.
C ***   fracol        : Rahmenfarbe
C ***
C ***   Modification:
C ***
C ***   19-SEP-1992 07:01:21.00         ML / UT Austin.  Ugly fix installed for
C ***                                   brain dead TEXTLK/SYMBLK.  Will BK ever
C ***                                   fix this silly limit???
C ***
C ******************************************************************************
C ***
        character*(*) xtext, ytext, head
        real l
        integer fracol
        logical xlab, ylab
        common /pltpar/ xscal, yscal, x0, y0, xc0, yc0, xc1, yc1, 
     >    xlab, ylab, xsiz, ysiz, headsz, symsiz

        external labini                 ! Fuer linker

	call pltsyn( .true. )
        call clip( xorg, yorg, xsize, ysize, xmin, xmax, ymin, ymax )

        call newpen( fracol )

        lx = idx( xtext )               ! [ char ]
        ly = idx( ytext )               ! [ char ]
        lh = max( 1, idx( head ) )      ! [ char ]

        if( xlab ) then
           call achsiz( xsiz )
           call achse( xorg, yorg, xtext(:lx), -lx, xsize, 0., xmin, 
     >       xinc, xmax )
        else
           call achse0( xorg, yorg, -lx, xsize, 0., xmin, 
     >       xinc, xmax )
        end if
        call achse0( xorg + xsize, yorg, -ly, ysize, 90., ymin, yinc, 
     >    ymax )
        call achse0( xorg,  yorg + ysize, lx, xsize, 0., xmin, xinc,
     >    xmax )
        if( ylab ) then
           call achsiz( ysiz )
           call achse( xorg, yorg, ytext(:ly), ly, ysize, 90., ymin, 
     >       yinc, ymax )
        else
           call achse0( xorg, yorg, ly, ysize, 90., ymin, 
     >       yinc, ymax )
        end if
C ***
C ***   This is the nonsense to make TEXTLK/SYMBLK not crash.  Won't give
C ***   correct results if control sequences (&) are at the `wrong' places
C ***
        l = 0.
        more_junk = min( lh, 80 )
        do junk = 1, lh, 80
           call textlk( d, al, headsz, head(junk:more_junk), d,
     >                                                more_junk - junk )
           l = l + al
           more_junk = min( more_junk + 80, lh )
        end do
        xx = xorg + dim(xsize,l)/2.
        yy = yorg + ysize + headsz
        more_junk = min( lh, 80 )
        do junk = 1, lh, 80
           call symblk( xx, yy, headsz, head(junk:more_junk), 0.,
     >                                                more_junk - junk )
           xx = 999.
           yy = 999.
           more_junk = min( more_junk + 80, lh )
        end do

	call pltsyn( .false. )
        end
        subroutine clip( xorg, yorg, xsize, ysize, xmin, xmax, ymin,
     >                   ymax )
C ***
C ******************************************************************************
C ***
C ***                    14-NOV-1992 01:22:47.83
C ***
C ***   Setzt Cliprahmen und Limits fr Achsen fuer Subroutine OVER.
C ***
C ***   xorg, yorg    : Koordinaten in cm der linken unteren Ecke des
C ***                   Kastens, relativ zum aktuellen Ursprung der Zeichnung
C ***   xsize, ysize  : Laenge der x- bzw. y-Achse in cm.
C ***   xmin, xmax    : Kleinster und groesster x-Wert in dargest. Einheiten
C ***   ymin, ymax    : Kleinster und groesster y-Wert in dargest. Einheiten
C ***
C ***   Wird benutzt von DIAGR und FRAME.
C ***
C ******************************************************************************
C ***
        implicit none

        real xorg, yorg, xsize, ysize, xmin, xmax, ymin, ymax

        logical xlab, ylab
        real xscal, yscal, x0, y0, xc0, yc0, xc1, yc1, xsiz
        real ysiz, hsiz, symsiz
        common /pltpar/ xscal, yscal, x0, y0, xc0, yc0, xc1, yc1, 
     >    xlab, ylab, xsiz, ysiz, hsiz, symsiz

        xscal = xsize/(xmax - xmin)     ! [ cm/usercoord. ]
        yscal = ysize/(ymax - ymin)     ! [ cm/usercoord. ]
        x0 = xorg - xmin*xscal          ! [ cm ]
        y0 = yorg - ymin*yscal          ! [ cm ]

        xc0 = xorg              ! Linke untere Ecke [ cm ]
        yc0 = yorg              ! Linke untere Ecke [ cm ]
        xc1 = xorg + xsize      ! Rechte obere Ecke [ cm ]
        yc1 = yorg + ysize      ! Rechte obere Ecke [ cm ]

        end
        subroutine over( x, y, n, dshlen )
C ***
C ******************************************************************************
C ***
C ***                    30-JUL-1988 14:22:02 
C ***
C ***   Plottet Kurve aus Arrays x,y mit n Punkten.
C ***
C ***   dshlen: Gestrichelte Kurve, Strichlaenge,
C ***            0: kontinuierlich
C ***           -n: Nummer fuer CENSYM-Symbol
C ***     >= 10000: step function style
C ***               with 10000   : continous lines and data points at 
C ***                              left side of steps,
C ***                    10000.1 : same but dashed lines with 1 mm long dashes
C ***                    10000.2 : 2 mm long dashes
C ***                    10001   : 1 cm long dashes
C ***                    10001.1 : 1.1 cm long dashes
C ***
C ***                    10100   : continous lines with data points at right
C ***                    11000   : continous lines with steps centered between
C ***                              data points
C ***                    10100.1 : right data points with 1mm dashed lines
C ***                    11000.1 : centered data points with 1mm dashed lines
C ***
C ***                    20000   : filled chart-like bars, points at left side
C ***                    21000   : filled bars, centered steps
C ***                    20100   : filled bars, points at right
C ***
C ***   Modification:
C ***
C ***   28-NOV-2002 22:51:21.00         ML / Stegaurach.  Moved plot of first
C ***                                   continous point into loop.  Was plotting
C ***                                   something if called with n.eq.0
C ***    2-DEC-2002 22:44:59.00         ML / Stegaurach.  Added step like curve
C ***                                   style.
C ***    3-DEC-2002 21:06:57.00         ML / Stegaurach.  dshlen -100 ==> 1000
C ***    5-DEC-2002 22:51:02.00         ML / Stegaurach.  dshlen ==> 10000 and
C ***                                   extension to left, right, and centered
C ***                                   bins.
C ***    7-DEC-2002 01:00:32.00         ML / Stegaurach.  Filled bins.
C ***    8-DEC-2002 23:07:59.00         ML / Stegaurach.  Optimize for points
C ***                                   outside of plot area.  Assumes data 
C ***                                   points are sorted.  Should speed up
C ***                                   cdash a lot but can't plot squares or
C ***                                   similar any longer.
C ***
C ******************************************************************************
C ***
        implicit none

C ***
C ***   Arguments
C ***
        integer n
        real x(n), y(n), dshlen
        integer color

C ***
C ***   Local variables
C ***
        integer i, istart, iend, inc, m
        real xcmin, xcmax, xswap
        logical asc
        character mist*4

C ***
C ***   Global variables
C ***
        logical xlab, ylab
        real xscal, yscal, x0, y0, xc0, yc0, xc1, yc1, 
     >       xsiz, ysiz, hsiz, symsiz
        common /pltpar/ xscal, yscal, x0, y0, xc0, yc0, xc1, yc1, 
     >                  xlab, ylab, xsiz, ysiz, hsiz, symsiz

C ***
C ***   Functions
C ***
        real xcm, ycm
        external xcm, ycm

C ***
C ***   Code
C ***
        mist = 'Over'
        go to 1

        entry ovr( x, y, n, dshlen, color )
C ***         ===
C ***
C ***   Just like over but with color.
C ***
C ***
        mist = 'Ovr'
        call newpen( color )

1       continue

        call derr( mist )

        if( n .le. 0 ) return                   ! no points -> give up

        if( dshlen .ge. 0 ) then                ! for anything but symbols
C ***
C ***      Find start and end points lying within drawing area
C ***
           asc = .true.                         ! check how data is sorted
           do i = 1, n - 1
              if( x(i) .ne. x(i+1) ) then
                 asc = x(i+1) .gt. x(i)
                 go to 2 ! exit loop
              end if
           end do 

2          continue
           xcmin = (xc0 - x0)/xscal    ! xmin in user coordinates
           xcmax = (xc1 - x0)/xscal    ! xmax in user coordinates
           if( xcmin .gt. xcmax ) then
              xswap = xcmax
              xcmax = xcmin
              xcmin = xswap
           end if

           if( asc ) then                       ! ascending data
              istart = 1
              iend = n
              inc = 1
           else                                 ! descending data
              istart = n
              iend = 1
              inc = -1
           end if
           do i = istart, iend, inc  ! find first left point inside drawing area
              if( x(i) .ge. xcmin ) then
                 if( asc ) then
                    istart = max( i - inc, istart )   ! take one point outside
                 else                                 ! of area
                    istart = min( i - inc, istart )
                 end if
                 go to 3 ! exit
              end if
           end do
3          continue
           do i = istart + inc, iend, inc      ! find right point inside area
              if( x(i) .gt. xcmax ) then
                 iend = i
                 go to 4 ! exit
              end if
           end do
4          continue
           if( .not.asc ) then                 ! swap for descending data points
              m = iend                         ! so we plot right to left again
              iend = istart                    ! and avoid increments to pass to
              istart = m                       ! subroutines
           end if
           m = iend - istart + 1               ! number of points in area 
        end if

        call pltsyn( .true. )
        call cplot( xc0, yc0, 997 )
        call cplot( xc1, yc1, 998 )

        if( dshlen .eq. 0. ) then
           call cplot( xcm(x(istart)), ycm(y(istart)), 3 )
           do i = istart + 1, iend
              call cplot( xcm(x(i)), ycm(y(i)), 2 )
           end do
        else if( dshlen .lt. 0 ) then
           do i = 1, n
              if( ( (xcm(x(i)) .le. xc1) .and. 
     >          ( xcm(x(i)) .ge. xc0 ) ) .and.
     >          ( ( ycm(y(i)) .le. yc1 ) .and.
     >          ( ycm(y(i)) .ge. yc0 ) ) ) 
     >          call censym( xcm(x(i)), ycm(y(i)), int(abs(dshlen)), 
     >             symsiz, symsiz, 0. )
           end do
        else if( dshlen .ge. 21000. ) then
C ***      filled `centered' bins
           call binplot( x(istart), y(istart), m, 
     >                   dshlen - 21000., 0, 1 )
        else if( dshlen .ge. 20100. ) then
C ***      filled bins with points at right
           call binplot( x(istart), y(istart), m,
     >                   dshlen - 20100., 1, 1 )
        else if( dshlen .ge. 20000. ) then
C ***      filled bins with points at left
           call binplot( x(istart), y(istart), m, 
     >                   dshlen - 20000., -1, 1 )
        else if( dshlen .ge. 11000. ) then
C ***      `centered' bins
           call binplot( x(istart), y(istart), m, 
     >                   dshlen - 11000., 0, 0 )
        else if( dshlen .ge. 10100. ) then
C ***      bins with points at right
           call binplot( x(istart), y(istart), m, 
     >                   dshlen - 10100., 1, 0 )
        else if( dshlen .ge. 10000. ) then
C ***      bins with points at left
           call binplot( x(istart), y(istart), m, 
     >                   dshlen - 10000., -1, 0 )
        else
           call cdash( xcm(x(istart)), ycm(y(istart)), 3, dshlen )
           do i = istart + 1, iend
              call cdash( xcm(x(i)), ycm(y(i)), 2, dshlen )
           end do
        end if  
        call pltsyn( .false. )

        end
        subroutine gposu( x, y, c )
C ***
C ******************************************************************************
C ***
C ***                    30-JUL-1988 14:22:09 
C ***
C ***   Liefert Position des Grafikcursors in dargestellten Einheiten.
C ***
C ******************************************************************************
C ***
        character c
        logical xlab, ylab
        common /pltpar/ xscal, yscal, x0, y0, xc0, yc0, xc1, yc1, 
     >    xlab, ylab, xsiz, ysiz, hsiz, symsiz

        call derr( 'Gposu' )

        call getpos( x, y, c )
        x = (x - x0)/xscal 
        y = (y - y0)/yscal 

        return
        end
        subroutine ptext( x, y, size, text )
C ***
C ******************************************************************************
C ***
C ***   Plot a text string.  Use with diagr system.
C ***
C ***   x,y     > R     position of lower left corner [user coordinates]
C ***   size    > R     character height [user coordinates]
C ***   text    > C     string to plot
C ***
C ***   Modifications:
C ***
C ***   21-NOV-1994 15:19:59.00         ML / IoA.  Created.
C ***   26-JAN-1996 16:07:20.00         ML / Bamberg.  Split calls to symblk to
C ***                                   make it work with original BK software.
C ***   19-JUN-1996 16:20:51.75         ML / Bamberg.  Added ptxt entry for more
C ***                                   flexibility.
C ***
C ******************************************************************************
C ***
        implicit none

C ***
C ***   Arguments
C ***
        real x, y, size
        character*(*) text
        integer loc, col, icoord        ! entry ptxt

C ***
C ***   Global variables
C ***
        logical xlab, ylab
        real xscal, yscal, x0, y0, xc0, yc0, xc1, yc1, 
     >       xsiz, ysiz, hsiz, symsiz
        common /pltpar/ xscal, yscal, x0, y0, xc0, yc0, xc1, yc1, 
     >                  xlab, ylab, xsiz, ysiz, hsiz, symsiz

C ***
C ***   Functions
C ***
        integer idx
        real xcm, ycm
        external xcm, ycm, idx

C ***
C ***   Local variables
C ***
        logical normx, normy, normsize
	integer status
        integer i, j, iloc, icol, junk, more_junk, lh
        real xx, yy, cmsize, al, d, l
        character coord*3, mist*5

        iloc = 1
        normx = .false.
        normy = .false.
        normsize = .false.
	mist = 'Ptext'
        go to 1

        entry ptxt( x, y, text, size, loc, col, icoord )
C ***
C ******************************************************************************
C ***
C ***   Plot a text string.  Use with diagr system.
C ***
C ***   x,y     > R     position of lower left corner [user/norm.dev. coord.]
C ***   text    > C     string to plot
C ***   size    > R     character height [user/norm.dev. coord.]
C ***   loc     > I     placement as in Mongo's putlabel:
C ***                   7 8 9     7----8----9   where these numbers define a
C ***                   4 5 6     4 Te 5 xt 6   box around/across Text and x/y
C ***                   1 2 3     1----2----3   is the corresponding location
C ***   col     > I     color
C ***   icoord  > I     3 figure units flag: xys for x, y, size, respectively
C ***                       0: user coordinates
C ***                   .ne.0: normalized device coordinates [0..1]
C ***                   e.g. 101: x in norm dev, y in user, size in norm dev.
C ***
C ***   Modifications:
C ***
C ***   21-NOV-1994 15:19:59.00         ML / IoA.  Created.
C ***   26-JAN-1996 16:07:20.00         ML / Bamberg.  Split calls to symblk to
C ***                                   make it work with original BK software.
C ***   19-JUN-1996 16:20:51.75         ML / Bamberg.  Added ptxt entry for more
C ***                                   flexibility.
C ***   21-JUN-1996 19:55:01.13         ML / Bamberg.  icoord added.
C ***
C ******************************************************************************
C ***
        iloc = loc
        call newpen( col )

	mist = 'Ptxt'
        write( coord, '(i3.3)', iostat = status ) icoord
        normx = coord(:1) .ne. '0'
        normy = coord(2:2) .ne. '0'
        normsize = coord(3:3) .ne. '0'


1       continue        ! for ptext entry

        call derr( mist )


C ***
C ***   Determine string length in cm
C ***
        if( normsize ) then
           cmsize = (yc1 - yc0)*size   ! size is fraction of frame height
        else
           cmsize = yscal*size         ! size is y user units
        end if

C ***
C ***   This is the nonsense to make TEXTLK/SYMBLK not crash.  Won't give
C ***   correct results if control sequences (&) are at the `wrong' places
C ***
        lh = max( 1, idx( text ) )      ! [ char ]
        l = 0.
        more_junk = min( lh, 80 )
        do junk = 1, lh, 80
           call textlk( d, al, cmsize, text(junk:more_junk), d,
     >                  more_junk - junk )
           l = l + al
           more_junk = min( more_junk + 80, lh )
        end do

C ***
C ***   Define placement
C ***
        if( normx ) then
           xx = x*(xc1 - xc0) + xc0  ! x is fraction of frame width
        else
           xx = xcm(x)               ! x is x user units
        end if

        if( normy ) then
           yy = y*(yc1 - yc0) + yc0  ! y is fraction of frame height
        else
           yy = ycm(y)               ! y is y user units
        end if

        if( iloc .ge. 7 ) then
           yy = yy - cmsize             ! y is top of line
        else if( iloc .ge. 4 ) then
           yy = yy - 0.5*cmsize         ! y is line center
        end if

        if( mod( iloc, 3 ) .eq. 2 ) then
           xx = xx - 0.5*l              ! text centered
        else if( mod( iloc, 3 ) .eq. 0 ) then
           xx = xx - l                  ! text flush right
        end if

C ***
C ***   Plot it (with Kalhoff fix)
C ***
        j = min( len(text), 80 )
        do i = 1, len(text), 80
           call symblk( xx, yy, cmsize, text(i:j), 0., j - i )
           xx = 999.
           yy = 999.
           j = min( j + 80, len(text) )
        end do

!       call symblk( xx, yy, cmsize, text, 0., len(text) )

        end
        subroutine derr( mist )
C ***
C ******************************************************************************
C ***
C ***            30-JUL-1988 14:22:18 
C ***
C ***   Error-Routine fuer diagr-System.
C ***
C ***	 5-JUL-1996 21:11:14.10		ML / Bamberg.  Mist parameter added
C ***
C ******************************************************************************
C ***
	character*(*) mist
        logical xlab, ylab
        common /pltpar/ xscal, yscal, x0, y0, xc0, yc0, xc1, yc1, 
     >    xlab, ylab, xsiz, ysiz, hsiz, symsiz

        if( xscal .eq. 4hErro ) then
           call pltend
           print *, '?DGRDNC: Illegal call of ', mist
           print *, '?DGRDNC: DIAGR or FRAME must be called first.'
           stop
        end if

        end
        function xcm(x)
C ***
C ******************************************************************************
C ***
C ***            30-JUL-1988 14:22:24 
C ***
C ***   Liefert aus Userkoordinaten Zentimeter fuer x-Koordinaten.
C ***   (DIAGR-System)
C ***
C ******************************************************************************
C ***
        logical xlab, ylab
        common /pltpar/ xscal, yscal, x0, y0, xc0, yc0, xc1, yc1, 
     >    xlab, ylab, xsiz, ysiz, hsiz, symsiz

        call derr( 'Xcm' )
        xcm = x*xscal + x0

        end
        function ycm(y)
C ***
C ******************************************************************************
C ***
C ***            30-JUL-1988 14:22:30 
C ***
C ***   Liefert aus Userkoordinaten Zentimeter fuer y-Koordinaten.
C ***   (DIAGR-System)
C ***
C ******************************************************************************
C ***
        logical xlab, ylab
        common /pltpar/ xscal, yscal, x0, y0, xc0, yc0, xc1, yc1, 
     >    xlab, ylab, xsiz, ysiz, hsiz, symsiz

        call derr( 'Ycm' )
        ycm = y*yscal + y0

        end
        function xcmc(x)
C ***
C ******************************************************************************
C ***
C ***   Liefert aus Userkoordinaten Zentimeter innerhalb Zeichenflaeche
C ***   fuer x-Koordinaten.
C ***   (DIAGR-System)
C ***
C ******************************************************************************
C ***
        implicit none
        real x, xcmc

        real xcm

C ***
C ***   Global variables
C ***
        logical xlab, ylab
        real xscal, yscal, x0, y0, xc0, yc0, xc1, yc1, 
     >       xsiz, ysiz, hsiz, symsiz
        common /pltpar/ xscal, yscal, x0, y0, xc0, yc0, xc1, yc1, 
     >                  xlab, ylab, xsiz, ysiz, hsiz, symsiz

        call derr( 'XcmC' )
        xcmc = max( min( xcm(x), xc1-0.25 ), xc0+0.25 )

        end
        function ycmc(y)
C ***
C ******************************************************************************
C ***
C ***   Liefert aus Userkoordinaten Zentimeter innerhalb Zeichenflaeche
C ***   fuer y-Koordinaten.
C ***   (DIAGR-System)
C ***
C ******************************************************************************
C ***
        implicit none
        real y, ycmc

        real ycm

C ***
C ***   Global variables
C ***
        logical xlab, ylab
        real xscal, yscal, x0, y0, xc0, yc0, xc1, yc1, 
     >       xsiz, ysiz, hsiz, symsiz
        common /pltpar/ xscal, yscal, x0, y0, xc0, yc0, xc1, yc1, 
     >                  xlab, ylab, xsiz, ysiz, hsiz, symsiz

        call derr( 'YcmC' )
        ycmc = max( min( ycm(y), yc1-0.25 ), yc0+0.25 )

        end
        subroutine fralab( xlab, ylab, xsiz, ysiz, hsiz, symsiz )
C ***
C ******************************************************************************
C ***
C ***   Setzt Zeichengroesse und Flag fuer Achsenbeschriftungen: 
C ***
C ***   XLAB / YLAB = .TRUE.    X/Y-Achse wird beschriftet mit Zahlen und Text
C ***                 .FALSE.   Nur Tickmarks zeichnen
C ***
C ***   XSIZ / YSIZ             Groesse der entsprechenden Achsenbeschriftungen
C ***                           in cm.
C ***
C ***   HSIZ                    Groesse der Ueberschrift [cm]
C ***
C ***   SYMSIZ                  Groesse der CENSYM-Zeichen
C ***
C ***
C ***    30-JUL-1988 13:45:00 
C ***
C ******************************************************************************
C ***
        logical xlab, ylab, xlabc, ylabc
        common /pltpar/ xscal, yscal, x0, y0, xc0, yc0, xc1, yc1, 
     >    xlabc, ylabc, xsizc, ysizc, hsizc, symsic

        xlabc = xlab
        ylabc = ylab
        xsizc = xsiz
        ysizc = ysiz
        hsizc = hsiz
        symsic = symsiz

        return
        end
        subroutine binplot( x, y, n, dshlen, ialign, ifill )
C ***
C ******************************************************************************
C ***
C ***   Make step function style plot.
C ***
C ***   x(n)    > R             Array with x values
C ***   y(n)    > R             Array with y values
C ***   n       > I             number of points in x and y
C ***   dshlen  > R             dash length.  continous if 0.
C ***   ialign  > I             Alignment of data points relative to bin:
C ***                           .lt. 0 : data point is on left side of bin
C ***                           .eq. 0 : data point is centered
C ***                           .gt. 0 : data point is on right side
C ***   ifill   > I             .ne. 0 : fill bins with solid color
C ***
C ***    2-DEC-2002 22:51:28.00         ML / Stegaurach.  Made.
C ***    5-DEC-2002 23:02:38.00         ML / Stegaurach.  Alignment added.
C ***
C ******************************************************************************

C ***
C ***   Arguments
C ***
        implicit none
        integer n
        real x(n), y(n), dshlen
        integer ialign, ifill

C ***
C ***   Local variables
C ***
        integer i
        real xc, yc, xcz, yczero, zero
        logical left, center, right, dashed, filled, asc

C ***
C ***   Global variables
C ***
        logical xlab, ylab
        real xscal, yscal, x0, y0, xc0, yc0, xc1, yc1, 
     >       xsiz, ysiz, hsiz, symsiz
        common /pltpar/ xscal, yscal, x0, y0, xc0, yc0, xc1, yc1, 
     >                  xlab, ylab, xsiz, ysiz, hsiz, symsiz

C ***
C ***   Functions
C ***
        real xcm, ycm, xcmc, ycmc
        external xcm, ycm, xcmc, ycmc

C ***
C ***   Code
C ***
        call derr( 'binplot' )

        if( n .lt. 2 ) return   ! not enough points -> give up

        asc = .true.
        do i = 1, n - 1         ! determine sort order of data
           if( x(i) .ne. x(i+1) ) then
              asc = x(i+1) .gt. x(i)
              go to 1 ! exit loop
           end if
        end do

1       continue

        zero = 0.       ! zero line, make argument at some point

        dashed = dshlen .gt. 1.e-6
        filled = ifill .ne. 0

        center = ialign .eq. 0
        if( asc ) then
           left = ialign .lt. 0
           right = ialign .gt. 0
        else                         ! descending data:
            left = ialign .gt. 0     ! left/right needs to be swapped
            right = ialign .lt. 0
        end if

        if( filled ) then
           yczero = ycmc(zero)
           if( left ) then
              xc = xcmc( x(1) )
              yc = ycmc( y(1) )
              do i = 2, n
                 xcz = xcmc( x(i) )
                 call pgrect( xc, xcz, yc, yczero )
                 xc = xcz
                 yc = ycmc( y(i) )
              end do
              call pgrect( xc, xcmc( x(n) + x(n) - x(n-1) ), 
     >                     yc, yczero )
           else if( right ) then
              xc = xcmc( x(1) - (x(2) - x(1)) )
              do i = 1, n
                 yc = ycmc( y(i) )
                 xcz = xcmc( x(i) )
                 call pgrect( xc, xcz, yc, yczero )
                 xc = xcz
              end do
           else if( center ) then
              xc = xcmc( x(1) - (x(2) - x(1))/2. )
              do i = 1, n - 1
                 yc = ycmc( y(i) )
                 xcz = xcmc( (x(i+1) + x(i))/2. )
                 call pgrect( xc, xcz, yc, yczero )
                 xc = xcz
              end do
              yc = ycmc( y(n) )
              xcz = xcmc( x(n) + (x(n) - x(n-1))/2. )
              call pgrect( xc, xcz, yc, yczero )
           end if
        else if( .not.dashed ) then
           if( left ) then      ! horizontal line first, then vertical
              xc = xcm( x(1) )
              yc = ycm( y(1) )
              call cplot( xc, yc, 3 )
              do i = 2, n
                 xc = xcm( x(i) )
                 call cplot( xc, yc, 2 )
                 yc = ycm( y(i) )
                 call cplot( xc, yc, 2 )
              end do
              call cplot( xcm( x(n) + x(n) - x(n-1) ), yc, 2 )
           else if( right ) then ! vertical line first, then horizontal
              xc = xcm( x(1) - (x(2) - x(1)) )
              yc = ycm( y(1) )
              call cplot( xc, yc, 3 )
              xc = xcm( x(1) )
              call cplot( xc, yc, 2 )
              do i = 2, n
                 yc = ycm( y(i) )
                 call cplot( xc, yc, 2 )
                 xc = xcm( x(i) )
                 call cplot( xc, yc, 2 )
              end do
           else if( center ) then
              xc = xcm( x(1) - (x(2) - x(1))/2. )
              yc = ycm( y(1) )
              call cplot( xc, yc, 3 )
              xc = xcm( (x(2) + x(1))/2. )
              call cplot( xc, yc, 2 )
              do i = 2, n - 1
                 yc = ycm( y(i) )
                 call cplot( xc, yc, 2 )
                 xc = xcm ( (x(i+1) + x(i))/2. )
                 call cplot( xc, yc, 2 )
              end do
              yc = ycm( y(n) )
              call cplot( xc, yc, 2 )
              xc = xcm( x(n) + (x(n) - x(n-1))/2. )
              call cplot( xc, yc, 2 )
           end if

        else
           if( left ) then      ! horizontal line first, then vertical
              xc = xcm( x(1) )
              yc = ycm( y(1) )
              call cdash( xc, yc, 3, dshlen )
              do i = 2, n
                 xc = xcm( x(i) )
                 call cdash( xc, yc, 2, dshlen )
                 yc = ycm( y(i) )
                 call cdash( xc, yc, 2, dshlen )
              end do
              call cdash( xcm( x(n) + x(n) - x(n-1) ), yc, 2, dshlen )
           else if( right ) then ! vertical line first, then horizontal
              xc = xcm( x(1) - (x(2) - x(1)) )
              yc = ycm( y(1) )
              call cdash( xc, yc, 3, dshlen )
              xc = xcm( x(1) )
              call cdash( xc, yc, 2, dshlen )
              do i = 2, n
                 yc = ycm( y(i) )
                 call cdash( xc, yc, 2, dshlen )
                 xc = xcm( x(i) )
                 call cdash( xc, yc, 2, dshlen )
              end do
           else if( center ) then
              xc = xcm( x(1) - (x(2) - x(1))/2. )
              yc = ycm( y(1) )
              call cdash( xc, yc, 3, dshlen )
              xc = xcm( (x(2) + x(1))/2. )
              call cdash( xc, yc, 2, dshlen )
              do i = 2, n - 1
                 yc = ycm( y(i) )
                 call cdash( xc, yc, 2, dshlen )
                 xc = xcm ( (x(i+1) + x(i))/2. )
                 call cdash( xc, yc, 2, dshlen )
              end do
              yc = ycm( y(n) )
              call cdash( xc, yc, 2, dshlen )
              xc = xcm( x(n) + (x(n) - x(n-1))/2. )
              call cdash( xc, yc, 2, dshlen )
           end if
        end if

        end
        
        block data labini

*       This is now in a separate file for stupid SunOS shareable images.
*       But this isn't stupid SunOS :-)
C ***
C ******************************************************************************
C ***
C ***   Initialisiert Werte fuer Achsenbeschriftung:
C ***
C ***   XLAB / YLAB = .TRUE.    Achsen werden beschriftet
C ***
C ***   XSIZ / YSIZ = 0.4       Beschriftungsgroesse ist 0.4 cm
C ***
C ***   HSIZ        = 0.4       Ueberschriftgroesse ist 0.4 cm
C ***
C ***   SYMSIZ      = 0.3       CENSYM-Zeichengroesse = 0.4 cm
C ***
C ***
C ***    30-JUL-1988 13:52:33 
C ***
C ******************************************************************************
C ***
        logical xlab, ylab
        common /pltpar/ xscal, yscal, x0, y0, xc0, yc0, xc1, yc1, 
     >    xlab, ylab, xsiz, ysiz, hsiz, symsiz

        data xlab, ylab, xsiz, ysiz, hsiz, symsiz / .true., .true., 0.4, 
     >    0.4, 0.4, 0.3 /

        data xscal /4hErro/
        end
