#include <sys/param.h> 
#include <sys/types.h>
#ifdef convex
#include <sys/dir.h>
#else
#include <dirent.h>
#endif
#include <ctype.h>
#ifdef __STDC__
#include <string.h>
#else
#include <strings.h>
#endif
#include <stdio.h>
#include <malloc.h>

/* This C routine, callable from FORTRAN, converts OLDNAME to NEWNAME by
 * the following rules:
 * If OLDNAME has a version number OLDNAME is returned regardless.
 * if STATUS="NEW", the maximum version is found and NEWNAME has a version
 *	of 1 greater.  If only a file with no version exists or no file
 *      exists at all NEWNAME has a version of 1.  This will never overwrite
 *	an existing file.
 * if STATUS="OLD", the maximum version is found and NEWNAME has the same
 *	version.  If only a file with no version exists or no file exists at
 *      all NEWNAME is a copy of OLDNAME (the last case should cause the
 *      OPEN statement to err).  This allows to access any existing file 
 *	regardless whether files with version numbers exist or not.
 * if STATUS="UNKNOWN", the maximum version is found and NEWNAME has the same
 *	version.  If only a file with no version exists NEWNAME is a copy of
 *      OLDNAME.  If no file exists at all than NEWNAME has a version of 1.
 *      This will always overwrite an existing file or create a new one with
 *      a version of 1.
 * Only the first character of status is relevant.  If STATUS is not "NEW" or
 * "UNKNOWN" it is assumed to be "OLD".  If the user has set the
 * environment variable INHIBIT_VERSION to anything then NEWNAME is a copy of
 * OLDNAME.  If something goes wrong NEWNAME is a copy of OLDNAME.  It is
 * assumed that there is enough room in new to hold the file name plus the
 * version number.
 * If called from C olen should be negative (see below).

 * version numbers come after periods (image.dst version 1 is image.dst.1)

 * Sam Southard, Jr.
 * Created: 29-Mar-1990
 *
 * Modifications:
 *
 *  7-MAY-1994 18:53:27.00	ML / IoA Cambs.  Made it work according to
 *				specs.  There were many bugs.  Changed version
 *				delimiter to . and env. variable to VERSIONS
 *				Clarified definitions in specs.  Default is
 *				now OLD - old version didn't work at all except
 *				for "OLD".
 *				Has not been tested with Fortran.
 *
 * 10-MAY-1994 16:11:25.00	ML / IoA Cambs.  Added relative version
 *                              numbers.
 *
 * 11-MAY-1994 12:09:33.00	ML / IoA Cambs.  Made sure output string is
 *				properly filled with spaces (Fortran) or null
 *				terminated (C).  Needs to be controled by
 *				calling sequence: make olen negative if called
 *				from C.
 *
 * 11-MAY-1994 20:45:26.00	ML / IoA Cambs.  Added context variable to
 *				avoid multiple reads of directory.
 *
 * 12-MAY-1994 19:45:55.00	ML / IoA Cambs.  Return status:
 *				 1: access ok for NEW/OLD/UNKNOWN resp.
 *				 0: don't know; would require call to access(2)
 *				-1: not ok
 *				-2: bad call (empty strings or something)
 *
 * 24-FEB-1995 16:57:51.00	ML / IoA.  Replaced rindex with ANSI strrchr
 */

#define min(a,b)	(((a) < (b)) ? (a) : (b))
#define max(a,b)	(((a) > (b)) ? (a) : (b))

#define OK 1
#define DUNNO 0
#define BAD -1
#define FATAL -2

int versname_(old,stat,new,context,olen,slen,nlen)
char *old;
char *stat;
char *new;
int *context;
long int olen;
long int slen;
long int nlen;
{
	char buf[MAXPATHLEN+1];
	char version[7];
	char *file, *ptr;
	char *ldot;
	char pad;
	int eos, relver=0, maxlen, versstart;
	int *maxvers;
	int status;
	int i;

	int *getversions();
	char *getenv();

	switch( *stat ) {
	case 'u':
	case 'U':
		status = OK;
		break;
	default:
		status = DUNNO;
	}

	if( olen < 0 ) {
	   olen = -olen;
           pad = '\0';
	}
	else pad = ' ';
	maxlen=min(olen,MAXPATHLEN);

	for (i=0 ; i < maxlen ; ++i) buf[i]=old[i];

	if( i == 0 || getenv("VERSIONS") == (char *)0 )
	{ /* no room to add a version number or user doesn't want versions */
		for ( i = 0; i < olen ; ++i) new[i]=old[i];
		return(DUNNO);
	}
	buf[i]='\0';

	while (i >= 0) /* find end of string (neither null (C) nor blank
                          (Fortran)) */
	{
	   if ( buf[i] != '\0' && !isspace(buf[i]) ) break;
	   buf[i--] = '\0';
	}
	if( i < 0 ) return(FATAL);  /* empty old or only spaces in old */

	if( (ptr = strrchr(buf,'.')) != NULL ) {	    /* find last dot */
           if( *(ptr+1) == '-' ) {      /* dot followed by '-'         */
	      relver++;                 /* assume relative vers. found */
              ldot = ptr;		/* remember where it was */
              ptr++;                    /* points to '-' now */
	   }
	   while (*ptr) {               /* all digits? */
	      if (!isdigit(*++ptr)) break;
	   }
	   /* something with a version, so give him what he wants */
	   if( !*ptr && !relver && *(ptr-1) != '.' ) {
	      for( i = 0; i < olen; i++ ) new[i] = old[i];
              if( !pad && i < nlen ) {
	         new[i] = pad;
              }
              else {
	         for( ; i < nlen; i++ ) {
	            new[i] = pad;
	         }
              }
	      return(status);
	   }

           if( relver ) {    /* really relative version? */
	      if( !*ptr ) {  /* ran through whole string */
	         if( *(ptr-1) == '-' ) relver--; /* last char == '-' =>     */
	      }                                  /* was no relative version */
              else relver--; /* stopped before end of string => no rel ver  */
	   }
           if( relver ) {    /* yes, really relative version */
	      *ldot = '\0'; 	       /* cut off relative version */
              relver = atoi( ldot+2 ); /* save as positive number */
              i = ldot - buf - 1;      /* length of file name for adding v# */
	   }
	}
	versstart = i + 1;	       /* position to add version number */
	for( i = 0; i < versstart; i++ ) new[i] = old[i];

	if ((file=strrchr(buf,'/')) == (char *)0) {
	   file = buf;
	   ptr = ".";  /* was a file only, use current directory */
	}
	else {
	   *file++ = '\0';
	   ptr = buf;
	}

	/* ptr now points to a null-terminated directory and file points to
	 * a null-termined file without any version number
	 */

	maxvers = getversions( ptr, file, context );

        maxvers++;	/* skip count */
	switch( *stat ) {
	case 'n':
	case 'N':
		if (*maxvers < 99998) (*maxvers)++;
	        *maxvers = max( *maxvers, 1 );
                status = OK;
	 	break;
	case 'u':
	case 'U':
	   	if( relver + 1 <= *(maxvers-1) ) *maxvers = maxvers[relver];
		else (*maxvers < 0) ? (*maxvers = 1) : (*maxvers += 1);
		*maxvers = *maxvers < 0 ? 1 : *maxvers;
		break;
	default: /* old */
	   	if( relver + 1 <= *(maxvers-1) ) *maxvers = maxvers[relver];
		else *maxvers = -1;
                status = *maxvers < 0 ? BAD : OK;
	}

	/* if version is 0, we should return (keep name) */
	/* the test is less than in case getversions returns a weird value */

	if( *maxvers <= 0 ) {
	   if( relver && *maxvers != 0 ) { /* something went wrong - add rest
					    * of old, i.e., relative version #
					    */
	      for( i = versstart; i < olen; i++ ) new[i] = old[i];
	   }
	   else i = versstart;
           if( !pad && i < nlen ) {
	      new[i] = pad;
           }
           else {
	      for( ; i < nlen; i++ ) {
	         new[i] = pad;
	      }
           }
	   return(status);
	}

#ifdef __STDC__
	i = sprintf( version, ".%d", *maxvers );
#else
	i = strlen( sprintf( version, ".%d", *maxvers ) );  /* non-ANSI */
#endif
	if( i + versstart > nlen ) return; /* no room for version number */
        maxlen = i + versstart;
	for( i--; i >= 0; i-- ) new[versstart+i] = version[i];
        i = maxlen;
        if( !pad && i < nlen ) {
	   new[i] = pad;
        }
        else {
	   for( ; i < nlen; i++ ) {
	      new[i] = pad;
	   }
        }

	return(status);
}

/* The getversions routine gets the maximum existing versions of the specified
 * file in the specified directory.  It is guaranteed to return a result in
 * the range -1 - 99999.  It returns -1 if the file or directory does not
 * exist.  It returns 0 if there is a file without a version number.
 * The returned array has the following layout:
 *
 *		getversions[0]   # of versions in array
 *		           [1]   max version (-1 if file does not exist)
 *		           [2]   second highest existing version
 *		           [.]
 *		           [.]
 *		           [.]
 *		           [n]   lowest existing version, can be 0 if there
 *				 is also a file without a version number
 */

int *
getversions(dir,file, context)
char *dir;
char *file;
int *context;
{
	DIR *dirp;

#ifdef convex
#define Dirent direct
#else
#define Dirent dirent
#endif

#define GROW 512
	struct Dirent *filep;
	static char olddir[MAXPATHLEN];
	static int *vers0;
	int *vers;

	int readd;
	int filelen;
	int namlen;
	static int name_free;
	static unsigned name_alloc;
	int filecount;
	unsigned more;
	static char *name_space;
	char *names, **namev;
	static char **namev0;
	char *fn;
	int nonnum;
	int i;

	int intcompare();
	DIR *opendir();
	struct Dirent *readdir();


	if( *context != 0 ) readd = strcmp( olddir, dir );
	else                readd = 1;

	if( readd ) {
	   if( name_free == 0 ) {
               name_free = GROW;
	       name_space = (char *)malloc( name_free );
               if( name_space == NULL ) {
	          fprintf( stderr, "Cannot get enough memory\n" );
	          exit(1);
	       }
               name_alloc = name_free;
               *context = (int)name_space;
	   }
	   vers = (int *)name_space;  /* so we can return something in case */
	   vers[0] = 1;               /* of trouble, e.g., opendir */
	   vers[1] = -1;

           filecount = 0;
           names = name_space;

	   if ((dirp=opendir(dir)) == (DIR *)0) return(vers);
	   while((filep=readdir(dirp)) != (struct Dirent *)0)
	   {
#if defined(__SVR4) || defined(Linux) || defined(__uxp__)
              filelen = strlen( filep->d_name );
#else
              filelen = filep->d_namlen;
#endif
              if( name_free <= filelen ) {
                 name_free += GROW*(filelen+1);
                 name_alloc += GROW*(filelen+1);
	         fn = (char *)realloc( name_space, name_alloc );
	         if( fn == NULL ) {
	            fprintf( stderr, "Cannot get enough memory\n" );
	            exit(1);
		 }
                 names += fn - name_space;  /* should the block have moved */
		 name_space = fn;
	      }

	      strcpy( names, filep->d_name );
	      names += filelen + 1;
              filecount++;
	      name_free -= filelen + 1;
	   }
           strcpy( olddir, dir );
	   closedir(dirp);

           if( (i=(names - name_space)%4) != 0 ) { /* move to word boundary */
	      names += 4 - i;
              name_free -= 4 - i;
	   }
           namev0 = (char **)names;  /* make pointer array start at
                                      * end of names block
                                      */
	   more = ( filecount + 1 )*sizeof(char **); /* pointers to file names */
	   more += ( filecount + 2 )*sizeof(int); /* version numbers */
	   more -= name_free;

           if( more > 0 ) {
              /* append space for pointer array and for version numbers */
              name_free += more;
              name_alloc += more;
	      fn = (char *)realloc( name_space, name_alloc );
	      if( fn == NULL ) {
	         fprintf( stderr, "Cannot get enough memory\n" );
	         exit(1);
	      }
              namev0 += (fn - name_space)/sizeof(char **);
	      name_space = fn;
           }
           for( namev = namev0, names = name_space; filecount; filecount-- ) {
	      *namev = names;
	      namev++;
              names += strlen(names) + 1;
           }
	   *namev++ = 0;
	   vers0 = (int *)namev; /* append version numbers to block */
           *context = (int)name_space;

	}

        vers = vers0;
	*vers++ = 1;
	*vers = -1;

	if ((filelen=strlen(file)) <= 0) return(vers0);


	for( namev = namev0; *namev; namev++ )
	{
		if (strncmp(file,*namev,filelen)) continue;
		if( *(*namev+filelen) == '\0' ) {
                   *vers++ = 0;		/* found requested file without v# */
		   continue;
		}
		else if( *(*namev+filelen) != '.' ) {
		   continue;		/* found something else without v# */
		}
		nonnum=0;		/* found something possibly with v# */
                namlen = strlen( *namev );
		for (i=filelen+1 ; i < namlen ; ++i)
			if (!isdigit(*(*namev+i)))
			{
				++nonnum;
				break;
			}
		if (nonnum) continue;
		if ((i=atoi((*namev)+filelen+1)) > 0 && i < 100000) {
                   *vers++ = i;
	        }
	}

	if( *(vers0+1) == -1 ) vers++;
	*vers0 = vers - vers0 - 1;  /* number of elements in vers array */
        qsort(vers0+1,*vers0,sizeof(int),intcompare);
	return(vers0);
}
#ifdef Michael_doesnt_know_what_he_is_doing
static int intcompare(i,j)
#else
int intcompare(i,j)
#endif
int *i, *j;
{
   return(*j - *i);
}
