/* This returns the position of the character before the period that separates
 * the version number from the file name.  If there is no version number it
 * returns the length of the file name.  Callable from C and Fortran.
 *
 * For C len_file must not include count of null byte and 1 must be subtracted
 * from result because of different array indexing (in other words it returns
 * the position of the period, which is where the null byte goes for
 * truncation).
 *
 * Example to cut off the version number:
 *
 * C:
 * 	char file[MAXPATHLEN];
 * 	file[ verstart_(file,strlen(file)) ] = '\0';
 *
 * Fortran:
 *	character*MAXPATHLEN file, file1
 *	integer verstart
 *	file1 = file(:verstart(file))
 * 
 *  9-MAY-1994 14:38:35.00	M. Lemke / IoA  Created.
 *
 * 10-MAY-1994 23:39:08.00	ML / IoA.  Added relative version number
 *
 */
#include <stdlib.h>
#include <stddef.h>
long
verstart_( file, len_file )
char *file;
long int len_file;
{
  	int i;

	if( getenv("VERSIONS") == NULL ) return( len_file );
	for( i = len_file - 1;
             i >= 0 && (file[i] == ' ' || file[i] == '\0');
             i-- ) {;}

	for( ; i >= 0 && isdigit(file[i]); i-- ) {;}

	if( i >= 0 && i <= len_file - 2 && 
            file[i] == '.' && isdigit(file[i+1]) ) return( (long)(i) );
                                                               /* ^ for 
                                            Fortran indexing - note not i-1! */
	else if( i >= 1 && i <= len_file - 2 &&
                 file[i] == '-' && file[i-1] == '.' && isdigit(file[i+1]) )
	                                           return( (long)(i-1) );
	else                                       return( len_file );
	    
}
