C
C ********************************************************************
C
 
      FUNCTION BCSS(ID,T,ANE,ILINE,ALAM,DLAM,DOPL)
C     ============================================
C
C     Absorption profile for four lines of He I, given by
C     Barnard, Cooper, Smith (1974) JQSRT 14, 1025 for the 4471 line;
C     Shamey (1969) PhD thesis, for other lines
C
C     Input: ID    - depth index
C            T     - temperature
C            ANE   - electron density
C            ILINE - index of the line ( = 1  for 4471,
C                                        = 2  for 4387,
C                                        = 3  for 4026,
C                                        = 4  for 4922)
C            ALAM  - wavelength (A)
C            DLAM  - offset from line centre (A)
C            DOPL  - Doppler width
C
C     Output: BCSS - profile coefficient in frequency units,
C                    normalized to unity
C ***
C ***	10-MAR-1997 16:16:15.43		ML / Bamberg.  Moved call to INIBCSS
C ***					to SFILOP to earlier detect missing
C ***					files .
C ***   16-APR-1997 23:13:06.00         ML / Stegaurach.  SAVE ipx, ipy, ipz
C ***                                   g77 gripes about them being
C ***                                   uninitialzed.  Maybe this helps.
C
      implicit none

      integer id, iline
      real t, ane, alam, dlam, dopl
      real bcss


      real prfhe1, dlmhe1, xnehe1
      integer nwlam
      COMMON/PROHE1/PRFHE1(50,4,8,3),DLMHE1(50,8,3),XNEHE1(8),
     *              NWLAM(8,4)
      real prf447, dlm447, xne447
      COMMON/PRO447/PRF447(80,4,7),DLM447(80,7),XNE447(7)
      real WLAM0(4),XT(4),XX(3),WX(3),YY(2),PP(2),ZZ(3),WZ(3)

      integer ilne, ipx, ipy, ipz, ix, iy, i0, i0x, i0z, jz, 
     >        n1x, n1y, n1z, n0x, n0y, n0z,
     >        nlst, nx, ny, nz, nt, ne
      real tl, anel, a, v, v1, d1, d2, w0, prf

	REAL alph(50)

      LOGICAL FIRST


      real extprf, h, wtot, yint


      SAVE FIRST, ipx, ipy, ipz

      DATA FIRST /.TRUE./
      DATA WLAM0 / 4471.50, 4387.93, 4026.20, 4921.93/
      DATA XT / 3.699, 4.000, 4.301, 4.602/
C
C     Initialize line data in common blocks
C
      IF (FIRST) THEN
!         CALL INIBCSS
         CALL TINT
         FIRST = .FALSE.
      ENDIF
C
C
      TL=LOG10(T)
      ANEL=LOG10(ANE)
      IF(ILINE.EQ.1.AND.ANEL.GE.XNE447(1)) GO TO 10
      IF(ILINE.NE.1.AND.ANEL.GE.XNEHE1(1)) GO TO 10
C
C     isolated line approximation for low electron densities
C
      A=WTOT(T,ANE,ID,ILINE)/DOPL
      V=ABS(DLAM)/DOPL
      V1=ABS(ALAM-4471.682)/DOPL
      BCSS=H(A,V)
      IF(ILINE.EQ.1) BCSS=(8.*BCSS+H(A,V1))/9.
      BCSS = BCSS / 1.772453851 / DOPL
      RETURN
C
C     otherwise, interpolation (or extrapolation) in tables
C
   10 NX=3
      NZ=3
      NY=2
      NT=4
      NE=8
      ILNE=ILINE-1
      IF(ILINE.EQ.1) NE=7
C
C     Interpolation in electron density
C
      DO 20 JZ=1,NE-1
         IPZ=JZ
	 IF(ILINE.EQ.1.AND.ANEL.LE.XNE447(JZ+1)) GO TO 30
	 IF(ILINE.NE.1.AND.ANEL.LE.XNEHE1(JZ+1)) GO TO 30
   20 CONTINUE
   30 N0Z=IPZ-NZ/2+1
      IF(N0Z.LT.1) N0Z=1
      IF(N0Z.GT.NE-NZ+1) N0Z=NE-NZ+1
      N1Z=N0Z+NZ-1
      DO 300 JZ=N0Z,N1Z
	 I0Z=JZ-N0Z+1
	 IF(ILINE.EQ.1) ZZ(I0Z)=XNE447(JZ)
	 IF(ILINE.NE.1) ZZ(I0Z)=XNEHE1(JZ)
C
C        Interpolation in temperature
C
	 DO 40 IX=1,NT-1
	    IPX=IX
	    IF(TL.LE.XT(IX+1)) GO TO 50
   40    CONTINUE
   50    N0X=IPX-NX/2+1
	 IF(N0X.LT.1) N0X=1
	 IF(N0X.GT.NT-NX+1) N0X=NT-NX+1
	 N1X=N0X+NX-1
	 DO 200 IX=N0X,N1X
	    I0X=IX-N0X+1
	    XX(I0X)=XT(IX)
C
C           Interpolation in wavelength
C
C           1. For delta lambda beyond tabulated values - special
C              extrapolation (Cooper's suggestion)
C
	    NLST=NWLAM(JZ,ILINE)
	    IF(ILINE.EQ.1) THEN
	       D1=DLM447(1,JZ)
	       D2=DLM447(NLST,JZ)
	       IF(DLAM.LT.D1) THEN
		  PRF=EXTPRF(DLAM,IX,ILINE,ZZ(I0Z),D1,PRF447(1,IX,JZ))
		  GO TO 150
                ELSE IF(DLAM.GT.D2) THEN
		  PRF=EXTPRF(DLAM,IX,ILINE,ZZ(I0Z),D2,
     *        	      PRF447(NLST,IX,JZ))
		  GO TO 150
               END IF
             ELSE
	       D1=DLMHE1(1,JZ,ILNE)
	       D2=DLMHE1(NLST,JZ,ILNE)
	       IF(DLAM.LT.D1) THEN
		  PRF=EXTPRF(DLAM,IX,ILINE,ZZ(I0Z),D1,
     *        	      PRFHE1(1,IX,JZ,ILNE))
		  GO TO 150
                ELSE IF(DLAM.GT.D2) THEN
		  PRF=EXTPRF(DLAM,IX,ILINE,ZZ(I0Z),D2,
     *        	      PRFHE1(NLST,IX,JZ,ILNE))
		  GO TO 150
               END IF
            END IF
C
C           normal linear interpolation in wavelength
C           (for 4471, linear interpolation in logarithms)
C
	    DO 60 IY=1,NLST-1
	       IPY=IY
	       IF(ILINE.EQ.1.AND.DLAM.LE.DLM447(IY+1,JZ)) GO TO 70
	       IF(ILINE.NE.1.AND.DLAM.LE.DLMHE1(IY+1,JZ,ILNE))
     *            GO TO 70
   60       CONTINUE
   70       N0Y=IPY-NY/2+1
	    IF(N0Y.LT.1) N0Y=1
	    IF(N0Y.GT.NLST-NY+1) N0Y=NLST-NY+1
	    N1Y=N0Y+NY-1
	    DO 100 IY=N0Y,N1Y
	       I0=IY-N0Y+1
	       IF(ILINE.EQ.1) YY(I0)=DLM447(IY,JZ)
	       IF(ILINE.EQ.1) PP(I0)=LOG(PRF447(IY,IX,JZ))
	       IF(ILINE.NE.1) YY(I0)=DLMHE1(IY,JZ,ILNE)
	       IF(ILINE.NE.1) PP(I0)=PRFHE1(IY,IX,JZ,ILNE)
  100      CONTINUE
	   IF(ILINE.NE.1) THEN
	      WX(I0X)=(PP(2)*(DLAM-YY(1))+PP(1)*(YY(2)-DLAM))/
     *                (YY(2)-YY(1))
	    ELSE
c             IF(YY(1).NE.0.) THEN
c                YY(1)=LOG(ABS(YY(1)))
c              ELSE
c                YY(1)=-8.
c             END IF
c             IF(YY(2).NE.0.) THEN
c                YY(2)=LOG(ABS(YY(2)))
c              ELSE
c                YY(2)=-8.
c             END IF
c             IF(DLAM.NE.0.) THEN
c                DLAM=LOG(ABS(DLAM))
c              ELSE
c                DLAM=-8.
c             END IF
             WX(I0X)=(PP(2)*(DLAM-YY(1))+PP(1)*(YY(2)-DLAM))/
     *                (YY(2)-YY(1))
             WX(I0X)=EXP(WX(I0X))
	   END IF
	   GO TO 200
  150      WX(I0X)=PRF
  200   CONTINUE
	WZ(I0Z)=YINT(XX,WX,TL)
  300 CONTINUE
      W0=YINT(ZZ,WZ,ANEL)
      BCSS=W0
      RETURN
      END
C
C********************************************************************
C

      SUBROUTINE INIBCSS
C     ==================
C
C     Initializes necessary arrays for evaluating the He I line
C     absorption profiles using data calculated by Barnard, Cooper
C     and Smith JQSRT 14, 1025, 1974 (for 4471)
C     or Shamey, unpublished PhD thesis, 1969 (for other lines)
C ***
C ***	 4-MAR-1997 20:58:24.87		ML / Bamberg.  Added readonly,
C ***                                   shared to OPEN statements
C ***	23-JUN-1997 14:48:07.91		ML / Bamberg.  OK! messages.
c ***   13-AUG-1997 21:57:18.33		ML / Bamberg.  readonly/shared
C ***                                   replaced with f90 action='read'
C ***	28-JAN-2002 21:11:03.00		ML / Stegaurach.  Allow
C ***					file names for BCSS_HE1 and BCSS_4471
C ***					with and without .DAT extension like
C ***					the other broadening tables.

C
      implicit none
      INCLUDE 'LINLTE.INC'
      INCLUDE 'FILES.INC'
C
      real prfhe1, dlmhe1, xnehe1
      integer nwlam
      COMMON/PROHE1/PRFHE1(50,4,8,3),DLMHE1(50,8,3),XNEHE1(8),
     *              NWLAM(8,4)
      real prf447, dlm447, xne447
      COMMON/PRO447/PRF447(80,4,7),DLM447(80,7),XNE447(7)

      integer i, ie, ie1, il, iln, ist, it, ne, nt, nwl
      real wl0, xne
      DATA NT /4/
C
C
C        read the Barnard, Cooper, Smith tables for He I 4471 line,
C        which have to be stored in file unit IH
C
         OPEN ( BCSS_4471 , FILE='BCSS_4471', STATUS='OLD'
*     >         , action= 'read' 
     >		, iostat = ist )
         if( ist .ne. 0 ) then
            OPEN ( BCSS_4471 , FILE='BCSS_4471.DAT', STATUS='OLD'
*     >            , action= 'read' 
     >		 )
         end if
        
         NE=7
         DO 10 IE=1,NE
            READ(BCSS_4471,501) IL,WL0,IE1,XNE,NWL
	    NWLAM(IE,1)=NWL
	    XNE447(IE)=LOG10(XNE)
            DO 10 I=1,NWL
               READ(BCSS_4471,502) DLM447(I,IE),
     *                   (PRF447(I,IT,IE),IT=1,NT)
   10    CONTINUE
         CLOSE ( BCSS_4471 )
	 write( stdout, * )  'BCSS_4471  OK!'
C
C        read Shamey's tables for He I 4387, 4026, and 4922 lines
C        which have to be stored in file unit IH
C
         OPEN ( BCSS_HE1 , FILE='BCSS_HE1', STATUS='OLD'
*     >         , action = 'read' 
     >		, iostat = ist )
         if( ist .ne. 0 ) then
            OPEN ( BCSS_HE1 , FILE='BCSS_HE1.DAT', STATUS='OLD'
*     >            , action = 'read' 
     >		 )
         end if
         NE=8
         DO 20 ILN=1,3
            DO 20 IE=1,NE
               READ(BCSS_HE1,501) IL,WL0,IE1,XNE,NWL
	       NWLAM(IE,ILN+1)=NWL
	       XNEHE1(IE)=LOG10(XNE)
               DO 20 I=1,NWL
                  READ(BCSS_HE1,*) DLMHE1(I,IE,ILN),
     *                       (PRFHE1(I,IT,IE,ILN),IT=1,NT)
   20    CONTINUE
         CLOSE ( BCSS_HE1 )
	 write( stdout, * )  'BCSS_HE1  OK!'
C
  501 FORMAT(/9X,I2,7X,F10.3,13X,I2,6X,E8.1,7X,I3/)
  502 FORMAT(5E10.2)
      RETURN
      END
C
C ********************************************************************
C
 
      FUNCTION WTOT(T,ANE,ID,ILINE)
C     =============================
C
C     Evaluates the total (electron + ion) impact Stark width
C     for four HeI lines
C     After Griem (1974); and Barnard, Cooper, Smith (1974) JQSRT 14,
C     1025 for the 4471 line
C
C     Input: T     - temperature
C            ANE   - electron density
C            ID    - depth index
C            ILINE - index of the line ( = 1  for 4471,
C                                        = 2  for 4387,
C                                        = 3  for 4026,
C                                        = 4  for 4922)
C     Output: WTOT - Stark width in Angstroms
C
      implicit none
      real t, ane
      integer id, iline
      real wtot

      INCLUDE 'LINLTE.INC'
      real ti0, ti1, ti2
      integer jt
      COMMON /TINTTT/ TI0(NDEPTH),TI1(NDEPTH),TI2(NDEPTH),JT(NDEPTH)

      integer i
      real alpha, ALPH0(4,4), f0, sig, W0(4,4), we, ALAM0(4)
      DATA ALPH0 / 0.107, 0.119, 0.134, 0.154,
     *             0.206, 0.235, 0.272, 0.317,
     *             0.172, 0.193, 0.218, 0.249,
     *             0.121, 0.136, 0.157, 0.184/
      DATA W0    / 1.460, 1.269, 1.079, 0.898,
     *             6.130, 5.150, 4.240, 3.450,
     *             4.040, 3.490, 2.960, 2.470,
     *             2.312, 1.963, 1.624, 1.315/
      DATA ALAM0 / 4471.50, 4387.93, 4026.20, 4921.93/
C
      I=JT(ID)
      ALPHA=(TI0(ID)*ALPH0(I,ILINE)+TI1(ID)*ALPH0(I-1,ILINE)+
     *      TI2(ID)*ALPH0(I-2,ILINE))*(ANE*1.E-13)**0.25
      WE=   (TI0(ID)*W0(I,ILINE)+TI1(ID)*W0(I-1,ILINE)+
     *      TI2(ID)*W0(I-2,ILINE))*ANE*1.E-16
      F0=1.884E19/ALAM0(ILINE)/ALAM0(ILINE)
      SIG=(4.32E-5*WE/SQRT(T)*F0/ANE**0.3333)**0.3333
      WTOT=WE*(1.+1.36/SIG*ALPHA**0.8889)
      RETURN
      END
 
C
C ********************************************************************
C
 
      FUNCTION EXTPRF(DLAM,IT,ILINE,ANEL,DLAST,PLAST)
C     ===============================================
C
C     Extrapolation in wavelengths in Shamey, or Barnard, Cooper,
C     Smith tables
C     Special formula suggested by Cooper
C
      implicit none
      real dlam, anel, dlast, plast
      integer it, iline
      real extprf

      real dlasta, d52, f, W0(4,4), we
      DATA W0    / 1.460, 1.269, 1.079, 0.898,
     *             6.130, 5.150, 4.240, 3.450,
     *             4.040, 3.490, 2.960, 2.470,
     *             2.312, 1.963, 1.624, 1.315/
C
      WE=W0(IT,ILINE)*EXP(ANEL*2.3025851)*1.E-16
      DLASTA=ABS(DLAST)
      D52=DLASTA*DLASTA*SQRT(DLASTA)
      F=D52*(PLAST-WE/3.14159/DLAST/DLAST)
      EXTPRF=(WE/3.14159+F/SQRT(ABS(DLAM)))/DLAM/DLAM
      RETURN
      END
 
C
C ********************************************************************
C
      SUBROUTINE TINT
C
C     LOGARITHMIC INTERPOLATION COEFFICIENTS FOR INTERPOLATION OF
C     TEMP(ID) TO THE VALUES  5000,10000,20000,40000
C
      implicit none
      INCLUDE 'LINLTE.INC'
      INCLUDE 'STRUCT.INC'
      real ti0, ti1, ti2
      integer jt
      COMMON /TINTTT/ TI0(NDEPTH), TI1(NDEPTH), TI2(NDEPTH), JT(NDEPTH)

      integer id, j
      real t, x
      real TT(4)
      DATA TT /3.699, 4.000, 4.301, 4.602/
C
      DO 10 ID=1,NMA
         T=LOG10(5040./THETA(ID))
         J=3
         IF(T.GT.TT(3)) J=4
         JT(ID)=J
         X=(TT(J)-TT(J-1))*(TT(J)-TT(J-2))*(TT(J-1)-TT(J-2))
         TI0(ID)=(T-TT(J-2))*(T-TT(J-1))*(TT(J-1)-TT(J-2))/X
         TI1(ID)=(T-TT(J-2))*(TT(J)-T)*(TT(J)-TT(J-2))/X
         TI2(ID)=(T-TT(J-1))*(T-TT(J))*(TT(J)-TT(J-1))/X
   10 CONTINUE
      RETURN
      END
C
C ********************************************************************
C
      FUNCTION YINT(XL,YL,XL0)
C
C     Quadratic interpolation routine
C
C     Input:  XL - array of x
C             YL - array of f(x)
C             XL0 - the point x(0) to which one interpolates
C
      implicit none
      real xl(3), yl(3), xl0
      real yint

      real a0, a1, a2, a3
      A0=(XL(2)-XL(1))*(XL(3)-XL(2))*(XL(3)-XL(1))
      A1=(XL0-XL(2))*(XL0-XL(3))*(XL(3)-XL(2))
      A2=(XL0-XL(1))*(XL(3)-XL0)*(XL(3)-XL(1))
      A3=(XL0-XL(1))*(XL0-XL(2))*(XL(2)-XL(1))
      YINT=(YL(1)*A1+YL(2)*A2+YL(3)*A3)/A0
      RETURN
      END
