	function bwb( temp, ne, iline, lam, dlam, vd, kma )
C ***
C ******************************************************************************
C ***
C ***	$Id: bwb.f,v 1.2 2004/03/03 23:53:51 ai26 Exp $
C ***
C ***	Interpolate BWB table.
C ***
C ***	We currently assume that there are 3 T points (10,000, 20,000, 40,000)
C ***	and 9 ne points (1e14,3e14,1e15,3e15,1e16,3e16,1e7,3e17,6e17) for
C ***	each line.
C ***
C ***
C ***	temp	r >	Temperature [K]
C ***	ne	r >	n_e	[cm^-3]
C ***	iline	i >	line index in table for which to get profile
C ***   lam	r >	central wavelength []
C ***	dlam	r >	[] distance from line center
C ***   vd	r >	Doppler velocity
C ***   kma     i >     line number in blend (standard LINE.DAT)
C ***	bwb	r <	the profile
C ***
C ***
C ***	22-OCT-1997 17:24:08.58		ML / Bamberg.  Made.
C ***	28-OCT-1997 20:50:29.42		ML / Bamberg.  Different treatment of
C ***					line center interpoaltion as an 
C ***					attempt to accout for poor wavelength
C ***					coverage of tables.
C ***   28-OCT-1997 21:59:12.22         ML / Stegaurach.  Hegriem stuff.
C ***	 4-NOV-1997 20:09:38.56		ML / Bamberg.  BWB.DAT Ok! message
C ***	 4-MAR-2004 00:32:13.01		ML / Stegaurach.  Try to cope with
C ***					readonly/shared via D lines.
C ***
C ******************************************************************************
C ***
	implicit none

C ***
C ***	Arguments
C ***
	real temp, ne
	integer iline, kma
	real lam, dlam, vd
	real bwb

C ***
C ***	Global variables
C ***
	include 'LINLTE.INC'
	include 'BWB.INC'

C ***
C ***	Functions
C ***
	integer hegriem
        real yint, exp10, linehe1
        external yint, hegriem, linehe1

C ***
C ***	Local variables
C ***
	integer i, it, ip, ine, jt, jne, iit, iine
        real x
	real log_ne, log_t
	real x1, x2, p1, p2, wt(3), wne(3)	! lambda interpolated profile
	real x3, x4, p3, p4, w1, w2
	real log_tbwb(3), log_nebwb(9)

	data log_tbwb /4., 4.301, 4.602/	! log T of table values
	data log_nebwb /14., 14.4771, 15., 15.4771, 16., 16.4771, 17.,
     >                  17.4771, 17.7782/	! log ne of table values

	save log_tbwb, log_nebwb

        exp10( x ) = exp( x*2.30259 )

C ***
C ***	Code
C ***

	log_ne = log10( ne )
	log_t = log10( temp )

C ***
C ***   Use Griem profile for low electron densities
C ***
        if( log_ne .lt. log_nebwb(1) - 0.1 ) then
	   i = hegriem( kma )
           if( i .gt. 0 ) then
              bwb = linehe1( temp, ne, i, lam, dlam, vd )
	   else
              go to  10 ! no Griem tables available, so try this stuff.
	   end if

           go to 99	! return
	end if

10      continue
	do i = 1, mnebwb - 1
	   if( log_ne .le. log_nebwb(i+1) ) then
              ine = i
	      go to 1
	   end if
	end do
	ine = mnebwb
1	continue
!        if( log_ne .lt. log_nebwb(1) .or. 
!     >      log_ne .gt. log_nebwb(mnebwb) ) print *, 'ne' ,ne,temp,dlam
!        if( ine .lt. 1 .or. ine .gt. mnebwb ) print *,ine,temp,ne,dlam
	ine = min( ine, mnebwb - 3 + 1 )

	do i = 1, mtbwb - 1
	   if( log_t .le. log_tbwb(i+1) ) then
	      it = i
	      go to 2
	   end if
	end do
	it = mtbwb
2	continue

!        if( log_t .lt. log_tbwb(1) .or. 
!     >      log_t .gt. log_tbwb(mtbwb) ) print *, 't ',ne,temp,dlam
!        if( it .lt. 1 .or . it .gt. mtbwb ) print *, it, temp, ne,dlam
	it = min( it, mtbwb - 3 + 1 )

        iine = 0
	do jne = ine, ine + 3 - 1
C ***
C ***	   ne interpolation
C ***
           iine = iine + 1
           iit = 0
           do jt = it, it + 3 - 1
C ***
C ***	      T interpolation
C ***
              iit = iit + 1
	      do ip = 1, ndlambwb(iline) - 1
C ***
C ***		 wavelength interpolation
C ***
	         if( dlam .le. dlambwb(ip+1,iline) ) then
                    p1 = profbwb( ip, jt, jne, iline )
                    p2 = profbwb( ip+1, jt, jne, iline )
                    x1 = dlambwb(ip,iline)
	            x2 = dlambwb(ip+1,iline)
                    if( x1*x2 .lt. 0. ) then
C ***
C ***                  Do `extrapolation' towards line center from either
C ***		       side [pairs (x1,x3) and (x2,x4)] and take minimum
C ***		       if there is no central point.
C ***		       
C ***
C ***	               Don't check for # of points -- should be enough
C ***
                       p3 = profbwb( ip-1, jt, jne, iline )
                       p4 = profbwb( ip+2, jt, jne, iline )
                       x3 = dlambwb(ip-1,iline)
	               x4 = dlambwb(ip+2,iline)
                       w1 = ((x2 - dlam)*p4 - (x4 - dlam)*p2)/
     >                           (x2 - x4)
                       w2 = ((x3 - dlam)*p1 - (x1 - dlam)*p3)/
     >                           (x3 - x1)
	               wt(iit) = min( w1, w2 )
	            else
C ***
C ***		       Ordinary interpolation
C ***
                       wt(iit) = ((x2 - dlam)*p1 - (x1 - dlam)*p2)/
     >                           (x2 - x1)
	            end if
!      if( temp .ge. 20000 )
!     >	print *, dlam, temp, ne, it, ine, p1, p2, x1, x2, wt(iit)
	            go to 3

	         end if
	      end do
3	      continue

	   end do
           wne(iine) = yint( log_tbwb(it), wt, log_t )    ! quadratic interpolation
	end do
        bwb = exp10( yint( log_nebwb(ine), wne, log_ne ) )
!        bwb = yint( log_nebwb(ine), wne, log_ne )

!	print *, bwb
99      continue
	end

	subroutine inibwb
C ***
C ******************************************************************************
C ***
C ***	Read BWB table.
C ***
C ***	We currently assume that there are 3 T points (10,000, 20,000, 40,000)
C ***	and 9 ne points (1e14,3e14,1e15,3e15,1e16,3e16,1e7,3e17,6e17) for
C ***	each line and that they are sorted.  ne is outer loop, T inner.
C ***
C ***
C ***	22-OCT-1997 17:19:35.17		ML / Bamberg.  Made.
C ***	29-OCT-1997 15:07:55.49		ML / Bamberg.  Renormalize profile
C ***
C ******************************************************************************
C ***
	implicit none

C ***
C ***	Global variables
C ***
	include 'LINLTE.INC'
	include 'FILES.INC'
	include 'BWB.INC'

C ***
C ***	Local variables
C ***
	integer ist, it, ine, ip, im, i, k
	real prof(mptbwb+1), dlam(mptbwb+1)
	real x1, x2, x3, x4, p1, p2, p3, p4, w1, w2, norm(mptbwb+1)
	character line*80

C ***
C ***	Code
C ***
	open( BWB_DAT, file = 'BWB', status = 'old', iostat = ist
d    >        , readonly, shared
     >      )
	if( ist .ne. 0 ) then
	   open( BWB_DAT, file = 'BWB.DAT', status = 'old'
d    >           , readonly, shared
     >         )
	end if

	nbwb = 0	! no line yet
        line = ' '
	do while( .true. )
	   do while( index(line, 'Wavelength') .eq. 0 )
	      read( BWB_DAT, '(a)', end = 1 ) line
	   end do
	   nbwb = nbwb + 1
	   if( nbwb .gt. mbwb ) then
	      write( stdout, * ) 'LINLTE>> Too many lines in BWB table.'
	      call exit
	   end if
	   read( line(index(line,'=')+1:), * ) lam0bwb(nbwb)
	   read( BWB_DAT, '(a)' ) line
	   read( line(index(line,'=')+1:), * ) ndlambwb(nbwb)
	   read( BWB_DAT, * ) (dlambwb(i,nbwb), i = 1, ndlambwb(nbwb))
	   if( ndlambwb(nbwb) .gt. mptbwb )  then
	      write( stdout, * ) 'LINLTE>>  Too many wavelength '//
     >                           'points in BWB table for line ',
     >                           lam0bwb(nbwb), ''
	      call exit
	   end if
	   k = 0
	   do i = 1, ndlambwb(nbwb)
	      k = k + 1
              dlam(k) = dlambwb(i,nbwb)
	      if( dlambwb(i,nbwb)*dlambwb(i+1,nbwb) .lt. 0. ) then
	         im = i
                 x1 = dlambwb(i,nbwb)
	         x2 = dlambwb(i+1,nbwb)
                 x3 = dlambwb(i-1,nbwb)
	         x4 = dlambwb(i+2,nbwb)
	         k = k + 1
	         dlam(k) = 0.
	      end if
	   end do
2	   continue
	   do ine = 1, mnebwb	! assume MNEBWB many ne values for all lines
	      do it = 1, mtbwb		! assume MTBWB many T values
	         read( BWB_DAT, '()' )	! skip ELECTRON/TEMP line
                 read( BWB_DAT, * ) (profbwb(ip,it,ine,nbwb), ip = 1,
     >                               ndlambwb(nbwb) )
	         k = 0
                 do ip = 1, ndlambwb(nbwb)
	            k = k + 1
	            if( ip .eq. im ) then
	               prof(k) = profbwb(ip,it,ine,nbwb)
	               k = k + 1

                       p1 = profbwb( ip, it, ine, nbwb )
                       p2 = profbwb( ip+1, it, ine, nbwb )
                       p3 = profbwb( ip-1, it, ine, nbwb )
                       p4 = profbwb( ip+2, it, ine, nbwb )
                       w1 = (x2*p4 - x4*p2)/(x2 - x4)
                       w2 = (x3*p1 - x1*p3)/(x3 - x1)
	               prof(k) = min( w1, w2 )

	            else
	               prof(k) = profbwb(ip,it,ine,nbwb)
	            end if
                 end do
	         k = ndlambwb(nbwb) + 1
                 call integ( dlam, prof, norm, k, 0. )
*	type *, nbwb, norm(k), ine, it, lam0bwb(nbwb)
	         k = 1
	         norm(k) = 1.
                 do ip = 1, ndlambwb(nbwb)
                    profbwb(ip,it,ine,nbwb) = log10(
     >                               profbwb(ip,it,ine,nbwb)/norm(k) )
                 end do
	      end do
	   end do
	end do
1	continue

	close( BWB_DAT )
	write( stdout, * ) 'BWB.DAT  OK!'

	end
