	subroutine h_data( k )
C ***
C ******************************************************************************
C ***
C ***	Fill in line data for Hydrogen lines.  For VCS theory compute
C ***	everything if ask to, for old stuff set only HY(K).
C ***
C ***	K	I  >	# of line in blend.
C ***
C ***	Modifications:
C ***
C ***	 15-JUN-1994 16:29:30.00	ML / IoA Cambridge.  Created.
C ***	  4-OCT-1994 20:43:00.00	ML / IoA.  Added option to find out
C ***					n_up and n_low from lambda
C ***	21-FEB-1995 15:11:33.00		ML / IoA.  Fixed C3 for Lyman series
C ***	14-SEP-1997 18:28:48.00		ML / Stegaurach.  New lineprof stuff.
C ***    7-NOV-1997 14:09:33.00         ML / Bamberg.  Correct wavelength
C ***                                   finding that allows series overlap.
C ***
C ******************************************************************************
C ***
	implicit none

C ***
C ***	Arguments
C ***
	integer k

C ***
C ***	Global variables
C ***
	include 'LINLTE.INC'
	include 'LINDAT.INC'
	include 'LINE.INC'

C ***
C ***	Local variables
C ***
	integer i, j, nl, nu, nl_max
        real d1, d2, l1, l2, n, m, dl, dll

C ***
C ***	Functions
C ***
	real lamair, h_vnm, h_fij, a, h_lam

	a(n,m) = 8.028619e9*m*m/(n*n)*(1./(m*m)-1./(n*n))**2*
     >                   h_fij(int(m),int(n))	! transition probability

	h_lam(i,j) = lamair( 2.997925e18/h_vnm(i, j) )


C ***
C ***	Code
C ***
	if( lprof(k) .eq. PVCS ) then
C ***
C ***	   for VCS
C ***
	   if( diu(k) .eq. 0. .or. lu(k) .eq. 0 ) then
	      if( lamk(k) .eq. 0. ) then
                 write( stdout, * ) 'LINLTE>> Need either wavelength',
     >                              ' or quantum numbers for VCS',
     >                              ' Hydrogen line.'
	         write( stdout, * ) '         Was processing line #', k
	         call exit
              end if
C ***
C ***         find n_low, get from series limit (will crash if lamk(k)
C ***         is below Lyman limit [who needs that?]):
C ***
	      do i = 1, 100
	         if( lamk(k) .lt. h_lam(i, 1000) ) then
                    go to 1
                 end if
              end do
1	      continue

	      nl_max = max( i - 1, 1 )

C ***
C ***	      and n_up:
C ***
              dll = 1e10		! something big for test below
	      do nl = 1, nl_max		! try all possible lower n
                 l2 = 1e10		! something big
                 do i = nl + 1, 100		! loop over all upper levels
                    l1 = h_lam( nl, i )
                    d1 = abs(lamk(k) - l1) 	! lambda is between these two
	            d2 = abs(lamk(k) - l2)
                    if( d1 .lt. d2 ) then	! find closer one
                        nu = i
	                dl = d1
                    else
                        nu = i - 1
	                dl = d2
                    end if
                    if( lamk(k) .le. l2 .and. lamk(k) .ge. l1 ) then
                       go to 2	! got it for this series; do next series
                    end if
                    l2 = l1
                 end do
2	         continue	! NU possible upper n,
				! DL distance to desired wavelength for that n

                 if( dl .lt. dll ) then	! if closer than last use this
	            lu(k) = nl		! lower n
                    diu(k) = nu		! upper n
                    dll = dl		! distance to desired wavelength
	         end if
              end do

              lamk( k ) = 0.	! and get real wavelength from below
           end if


	   if( lamk(k) .eq. 0. ) then
	      lamk( k ) = h_lam( lu(k), int(diu(k)) )
	   end if

	   if( chik(k) .eq. 0. ) then
	      chik(k) = 13.595*( 1. - 1./lu(k)/lu(k) )
	   end if

	   if( gflg(k) .eq. 0. ) then
	      gflg( k ) = log10( 2.*lu(k)*lu(k)*
     >                           h_fij( lu(k), int(diu(k)) ) )
	   end if

	   if( c4lg(k) .eq. 0. ) then
C ***
C ***	      resonance broadening, compute C3 = e^2*f_1j/(4*pi*m_e*nu_1j)
C ***
C ***	                23.0712e-20/(4.*3.1415926535*9.10956e-28)

              if( lu(k) .gt. 1 ) then
C ***
C ***            Balmer series and higher
C ***
	         c4lg( k ) = 2.01541e7*( h_fij( 1, lu(k) )/
     >                                   h_vnm( 1, lu(k) ) +
     >                                   h_fij( 1, int(diu(k)) )/
     >                                   h_vnm( 1, int(diu(k)) ) )
	      else
C ***
C ***            Lyman series
C ***
	         c4lg( k ) = 2.01541e7*( h_fij( 1, int(diu(k)) )/
     >                                   h_vnm( 1, int(diu(k)) ) )
              end if
	      c4lg( k ) = -log10( c4lg(k) )
	   end if

	   if( rad(k) .le. -0.001 ) then
C ***
C ***	      radiative broadening [10^8 sec^-1]
C ***
	      rad( k ) = 0.
	      do i = lu(k) - 1, 1, -1
		 rad( k ) = rad( k ) + a( real(lu(k)), real(i) )
	      end do
	      do i = int(diu(k)) - 1, 1, -1
		 rad( k ) = rad( k ) + a( diu(k), real(i) )
	      end do
              rad( k ) = 1.e-8*rad( k )

	   end if

	end if
	end
