        function hlinop( lam, t, pele )
        implicit none
C ***
C ******************************************************************************
C ***
C ***	Computes crummy approximation for H line opacity.  Stolen from ATLAS9.
C ***
C ***	lam:	in	wavelength [] at which to compute opacity
C ***	t:	in	temperature [K]
C ***	pele:	in	electron pressure [dyn/cm^2]
C ***	hlinop:	out	opacity
C ***
C ***
C ***	calls STARK and AHYLBF
C ***
C ***	22-FEB-1994 18:43:37.00		ML / IoA
C ***
C ***	Modifications:
C ***
C ***	23-FEB-1994 18:16:25.00		ML / IoA  Added conversion from air
C ***                                   wavelengths to vacuum.
C ***
C ***	 8-JUN-1994 16:54:15.00		ML / IoA  Changed to function.
C ***
C ***	11-AUG-1997 20:19:56.34		ML / Bamberg.  Changed frequency of
C ***					Lyman edge.  Got wrong wavelength for
C ***					Balmer jump.
C ***
C ***	19-MAY-1998 17:04:17.38		ML / Bamberg.  Changed ionization
C ***					energy to be the same as in ATMOS.
C ***					Fixed div/0 bug for mfreq calculation.
C ***
C ******************************************************************************
C ***

C ***
C ***	Arguments
C ***
	real lam, t, pele, hlinop


C ***
C ***	local variables
C ***
	integer m, m1, m2, mlast, mfreq, n
	real ehvkt, freq, kt, ktev, lam3, ne, bolt(4), oldt

	real f_ly
	parameter( f_ly = 3.2889214E+15 )	! frequency of Lyman edge
!	parameter( f_ly = 3.28805e15 )	! frequency of Lyman edge


C ***
C ***	Functions
C ***
	real stark, ahylbf, airlam

	save oldt, kt, ktev, ne, ehvkt, mlast, bolt
	data oldt /-5./

C ***
C ***	Code
C ***
        hlinop = 0.
	freq = 2.997925e18/airlam(lam)
        n = sqrt( f_ly/freq )
        if( n .le. 0 .or. n .gt. 4 ) then
	   return
        else if( n .eq. 1 .and. freq .lt. 2.e15 ) then
           return
        else if( n .eq. 2 .and. freq .lt. 4.44e14 ) then
	   return
        end if


	if( t .ne. oldt ) then
	   kt = 1.38063e-16*t
	   ktev = kt/1.602192e-12
	   ne = pele/kt
	   ehvkt = exp( -freq/kt*6.6262e-27 )

           mlast = 1100./ne**.133333333
           do 1 m = 1, 4
              bolt( m ) = exp( -(13.6059 - 13.6059/(m*m))/ktev )*
     >                    2.*m*m
1          continue
	   oldt = t
	end if
         
        if( (f_ly/(n*n) - freq) .eq. 0. ) then
	   mfreq = mlast + 1	! make it take the ahylbf branch below
	else
           mfreq = sqrt( f_ly/(f_ly/(n*n) - freq) )
	end if
        m1 = mfreq
        m2 = m1 + 1
        m1 = max( m1, n + 1 )
        if( m1 .gt. 6 ) then
           if( m1 .gt. mlast ) then
	      lam3 = airlam(lam)
	      lam3 = lam3*lam3*lam3
              hlinop = ahylbf( 1., lam3, n )*
     >                 ( 1. - ehvkt )*bolt( n )
	      return

           else
              m1 = m1 - 1
              m2 = m2 + 3
              if( n .eq. 4 .and. m1 .le. 8 ) then
                 hlinop = stark( 3, 4, freq, t, pele )*( 1. - ehvkt )*
     >                    bolt( 3 )
              end if
	   end if
	end if

        do 6 m = m1, m2
C ***   
C          ASSUMING FREQ APROXIMATELY FREQNM
C ***   
           hlinop = hlinop + stark( n, m, freq, t, pele )*
     >              ( 1. - ehvkt )*bolt(n)
6       continue
        end
