* All sorts of routines to fit and evaluate Legendre polynomials, and to
* convert coefficients to those of plain polynomials
	SUBROUTINE FITLPOLY(NPT,X,Y,SCALE,NCOEFF,COEFF)
* X and Y are data arrays with NPT points to fit.
* SCALE receives XMIN and XMAX, and returns A and B.
* where Z = A * (X - B)
* A polynomial of degree NCOEFF-1 is fit and the coefficients are
* returned in COEFF.
	PARAMETER( MAXFIT=8 )
	REAL*8 COV(MAXFIT*MAXFIT), VEC(MAXFIT)
	REAL*8 COEFF(*)
	REAL*8 A, B, Z, LPI, DET
	REAL*4 X(NPT), Y(NPT), SCALE(*)

	A = 2.0D0 / (SCALE(2) - SCALE(1))
	B = 0.5D0 * (SCALE(2) + SCALE(1))

	DO 5 J = 1,NCOEFF
5	VEC(J) = 0
	DO 6 I = 1,NCOEFF*NCOEFF
6	COV(I) = 0

	DO 10 N = 1,NPT
	Z = A * (DBLE(X(N)) - B)

	DO 20 J = 1,NCOEFF
	COEFF(J) = LPI(J,Z)
	VEC(J) = VEC(J) + COEFF(J) * Y(N)
	DO 20 I = 1,J
20	COV((J-1)*NCOEFF+I) = COV((J-1)*NCOEFF+I) + COEFF(I) * COEFF(J)

10	CONTINUE

	DO 30 J = 2,NCOEFF
	DO 30 I = 1,J-1
30	COV((I-1)*NCOEFF+J) = COV((J-1)*NCOEFF+I)

	CALL INVERT(NCOEFF,COV,COEFF,DET)
	IF(DET.EQ.0) THEN
	    WRITE(6,*) 'FITLPOLY: singular covariance matrix'
	    RETURN	
	ENDIF

	DO 40 J = 1,NCOEFF
	COEFF(J) = 0
	DO 40 I = 1,NCOEFF
40	COEFF(J) = COEFF(J) + COV((J-1)*NCOEFF+I) * VEC(I)

	SCALE(1) = A
	SCALE(2) = B

	RETURN
	END

	REAL FUNCTION LPOLY(X,SCALE,NCOEFF,COEFF)

	REAL*8 COEFF(*)
	REAL*8 TEMP, Z, LPI
	REAL*4 SCALE(*)

	Z = SCALE(1) * (X - SCALE(2))
	TEMP = 0
	DO 10 I = 1,NCOEFF
10	TEMP = TEMP + COEFF(I) * LPI(I,Z)
	LPOLY = TEMP
	RETURN
	END

	SUBROUTINE FIT2DLPOLY(NPT,X,Y,Z,SCALE,NCX,NCY,COEFF)
* X, Y and Z are data arrays with NPT points to fit.
* SCALE receives XMIN, XMAX, YMIN and YMAX and returns AX, BX, AY, BY
* where X' = AX * (X - BX)  and Y' = AY * (Y - BY)
* A two-dimensional polynomial of degree NCX-1 in X and NCY-1 in Y 
* is fit and the coefficients are returned in COEFF.
	PARAMETER( MAXFIT=8 )
	REAL*8 COV(MAXFIT*MAXFIT*MAXFIT*MAXFIT), VEC(MAXFIT*MAXFIT)
	REAL*8 COEFF(*)
	REAL*8 AX, BX, AY, BY, XSC, YSC, LPI, DET
	REAL*4 X(*), Y(*), Z(*), SCALE(*)

	AX = 2.0D0 / (SCALE(2) - SCALE(1))
	BX = 0.5D0 * (SCALE(2) + SCALE(1))
	AY = 2.0D0 / (SCALE(4) - SCALE(3))
	BY = 0.5D0 * (SCALE(4) + SCALE(3))

	NFIT = NCX*NCY

	DO 5 J = 1,NFIT
5	VEC(J) = 0
	DO 6 I = 1,NFIT*NFIT
6	COV(I) = 0

	DO 10 N = 1,NPT
	XSC = AX * (X(N) - BX)
	YSC = AY * (Y(N) - BY)

	DO 20 J = 1,NFIT
	JX = (J-1)/NCY + 1
	JY = MOD(J-1,NCY) + 1
	COEFF(J) = LPI(JX,XSC) * LPI(JY,YSC)
	VEC(J) = VEC(J) + COEFF(J) * Z(N)
	DO 20 I = 1,J
20	COV(I+(J-1)*NFIT) = COV(I+(J-1)*NFIT) + COEFF(I) * COEFF(J)

10	CONTINUE

	DO 30 J = 2,NFIT
	DO 30 I = 1,J-1
30	COV(J+(I-1)*NFIT) = COV(I+(J-1)*NFIT)

	CALL INVERT(NFIT,COV,COEFF,DET)
	IF(DET.EQ.0) THEN
	    WRITE(6,*) 'FIT2DLPOLY: singular covariance matrix'
	    RETURN	
	ENDIF

	DO 40 J = 1,NFIT
	COEFF(J) = 0
	DO 40 I = 1,NFIT
40	COEFF(J) = COEFF(J) + COV(I+(J-1)*NFIT) * VEC(I)

	SCALE(1) = AX
	SCALE(2) = BX
	SCALE(3) = AY
	SCALE(4) = BY

	RETURN
	END

	REAL FUNCTION LPOLY2D(X,Y,SCALE,NCX,NCY,COEFF)
* The desired fit has NCX coefficients in X and NCY in Y.
	REAL*8 COEFF(*), XSC, YSC, LPI, TEMP
	REAL*4 SCALE(*) 

	NFIT = NCX*NCY

	XSC = SCALE(1) * (X - SCALE(2))
	YSC = SCALE(3) * (Y - SCALE(4))

	TEMP = 0

	DO 20 J = 1,NFIT
	JX = (J-1)/NCY + 1
	JY = MOD(J-1,NCY) + 1
20	TEMP = TEMP + COEFF(J) * LPI(JX,XSC) * LPI(JY,YSC)

	LPOLY2D = TEMP

	RETURN
	END

	REAL FUNCTION POLY2D(X,Y,NCX,NCY,COEFF)
* The desired fit has NCX coefficients in X and NCY in Y.
	REAL*8 COEFF(*), T1, T2, DX, DY

	DX = DBLE(X)
	DY = DBLE(Y)

	T2 = 0

	DO J = NCX*NCY,NCY,-NCY
	    T1 = 0
	    DO I = J,J-NCY+1,-1
	    	T1 = T1 * DY + COEFF(I)
	    ENDDO
	    T2 = T2 * DX + T1
	ENDDO

	POLY2D = T2

	RETURN
	END

	FUNCTION LPI(I,Z)
* Legendre polynomial of order i-1
	REAL*8 LPI
	INCLUDE 'LEGENDRE.INC'
	GOTO(1,2,3,4,5,6,7,8) I
1	LPI = LP0(Z)
	RETURN
2	LPI = LP1(Z)
	RETURN
3	LPI = LP2(Z)
	RETURN
4	LPI = LP3(Z)
	RETURN
5	LPI = LP4(Z)
	RETURN
6	LPI = LP5(Z)
	RETURN
7	LPI = LP6(Z)
	RETURN
8	LPI = LP7(Z)
	RETURN
	END

	FUNCTION EVALEG(ORDER,X,SCALE,C)
	REAL*8 C(*)
	REAL*4 SCALE(2)
	INTEGER*4 ORDER
* Evaluates sum ( C(i) * LPi(a*(x-b)) )
* Legendre polynomials 0 - 7
	INCLUDE 'LEGENDRE.INC'

	Z = SCALE(1) * (X - SCALE(2))
	NCOEFF = ORDER + 1
	IF(NCOEFF.LT.1.OR.NCOEFF.GT.8) THEN
	    WRITE(6,*) ' EVALEG: wrong number of coeffs',NCOEFF
            EVALEG = -1.	! to make Fortran shut up
	    RETURN
	ENDIF

	GOTO(1,2,3,4,5,6,7,8) NCOEFF
1	EVALEG = C(1)*LP0(Z)
	RETURN
2	EVALEG = C(1)*LP0(Z) + C(2)*LP1(Z)
	RETURN
3	EVALEG = C(1)*LP0(Z) + C(2)*LP1(Z) + C(3)*LP2(Z)
	RETURN
4	EVALEG = C(1)*LP0(Z) + C(2)*LP1(Z) + C(3)*LP2(Z) +
     1	C(4)*LP3(Z)
	RETURN
5	EVALEG = C(1)*LP0(Z) + C(2)*LP1(Z) + C(3)*LP2(Z) +
     1	C(4)*LP3(Z) + C(5)*LP4(Z)
	RETURN
6	EVALEG = C(1)*LP0(Z) + C(2)*LP1(Z) + C(3)*LP2(Z) +
     1	C(4)*LP3(Z) + C(5)*LP4(Z) + C(6)*LP5(Z)
	RETURN
7	EVALEG = C(1)*LP0(Z) + C(2)*LP1(Z) + C(3)*LP2(Z) +
     1	C(4)*LP3(Z) + C(5)*LP4(Z) + C(6)*LP5(Z) +
     1	C(7)*LP6(Z)
	RETURN
8	EVALEG = C(1)*LP0(Z) + C(2)*LP1(Z) + C(3)*LP2(Z) +
     1	C(4)*LP3(Z) + C(5)*LP4(Z) + C(6)*LP5(Z) +
     1	C(7)*LP6(Z) + C(8)*LP7(Z)
	RETURN
	END

	SUBROUTINE POLYC(NC,SCALE,COEFF,PCOEFF)
* Routine to convert coefficients of a 1-d Legendre polynomial to 
* coefficients of a plain polynomial
* The NC coefficients are stored with the low order coefficient first
	REAL*8 COEFF(*), PCOEFF(*), SC(2), TEMP
	REAL*4 SCALE(2)
	INCLUDE 'LEGENDRE.INC'

	SC(1) = DBLE(SCALE(1))
	SC(2) = DBLE(SCALE(2))

	DO K = 0,NC-1
	    TEMP = 0
	    DO I = K,NC-1
	    	TEMP = TEMP + COEFF(I+1) * LCOEFF(K+1,I+1)
	    ENDDO
	    PCOEFF(K+1) = TEMP
	ENDDO
	DO K = 0,NC-1
	    TEMP = 0
	    DO I = K,NC-1
	    	TEMP = TEMP + PCOEFF(I+1) * SC(1)**I * 
     1	    	(-SC(2))**(I-K) * DFLOAT(IBC(K,I))
	    ENDDO
	    PCOEFF(K+1) = TEMP
	ENDDO

	RETURN
	END

	SUBROUTINE POLYCC(NX,NY,SCALE,COEFF,PCOEFF)
* Routine to convert coefficients of a 2-d Legendre polynomial to 
* coefficients of a plain polynomial
* The coefficients are stored as NX polynomials of order NY-1.
* Low order coefficient comes first, and the low order coefficient
* of the x polynomial is obtained by evaluating the first y polynomial.
	REAL*8 COEFF(*), PCOEFF(*), XSC(2), YSC(2), TEMP
	REAL*4 SCALE(4)
	INCLUDE 'LEGENDRE.INC'

	XSC(1) = DBLE(SCALE(1))
	XSC(2) = DBLE(SCALE(2))
	YSC(1) = DBLE(SCALE(3))
	YSC(2) = DBLE(SCALE(4))

	DO 10 J = 0,NX-1
	DO 11 K = 0,NY-1
	TEMP = 0
	DO 12 I = K,NY-1
12	TEMP = TEMP + COEFF(J*NY+I+1) * LCOEFF(K+1,I+1)
11	PCOEFF(J*NY+K+1) = TEMP
	DO 13 K = 0,NY-1
	TEMP = 0
	DO 14 I = K,NY-1
14	TEMP = TEMP + PCOEFF(J*NY+I+1) * YSC(1)**I * (-YSC(2))**(I-K) *
     1	DFLOAT(IBC(K,I))
13	PCOEFF(J*NY+K+1) = TEMP
10	CONTINUE

	DO 20 J = 0,NY-1
	DO 21 K = 0,NX-1
	TEMP = 0
	DO 22 I = K,NX-1
22	TEMP = TEMP + PCOEFF(I*NY+J+1) * LCOEFF(K+1,I+1)
21	PCOEFF(K*NY+J+1) = TEMP
	DO 23 K = 0,NX-1
	TEMP = 0
	DO 24 I = K,NX-1
24	TEMP = TEMP + PCOEFF(I*NY+J+1) * XSC(1)**I * (-XSC(2))**(I-K) *
     1	DFLOAT(IBC(K,I))
23	PCOEFF(K*NY+J+1) = TEMP
20	CONTINUE

	RETURN
	END

	FUNCTION IBC(M,N)
* Returns the (m n) binomial coefficient
	IBC = 1
	DO I = M+1,N
	    IBC = IBC * I
	ENDDO
	DO I = 2,N-M
	    IBC = IBC / I
	ENDDO
	RETURN
	END
