	subroutine linfit( lamobs, mobs, a, nterms, deriv, yfit,
     >                     der_calc )
	implicit none
C ***
C ******************************************************************************
C ***
C ***	Function to compute synthetic spectrum and derivatives for arbitrary
C ***   composition, v sini, xi mIAcro at wavelength points of an observed
C ***	spectrum.
C ***
C ***	Arguments:
C ***
C ***	LAMOBS	 > I	Wavelength of observation
C ***	MOBS	 > I	# of points in LAMOBS
C ***	A	 > R	Composition/v sini etc. to compute spectrum for.
C ***	NTERMS	 > I	# of elements in A.
C ***	DERIV	 < R	partial derivatives for each wavelength point LAMOBS.
C ***	YFIT	>< R	computed spectrum
C ***	DER_CALC > L	compute derivatives but no spectrum if .TRUE.
C ***
C ***	 9-JUN-1992 06:14:16.55
C ***	26-NOV-1994 15:23:06.00		ML / IoA.  call plotout(3) instead
C ***					of over if boa(1)
C ***	 7-MAY-1996 13:07:43.36		ML / Bamberg.  Fixed dimension mismatch 
C ***					of deriv (nobs instead of ipkt).
C ***
C ******************************************************************************
C ***

	include 'LINLTE.INC'

C ***
C ***	Arguments
C ***
	logical der_calc
	integer mobs, nterms
	real lamobs( mobs ), a( nterms ), yfit( mobs ),
     >       deriv( nobs, nfit )

C ***
C ***	Global variables
C ***
	include 'PROFIL.INC'
	include 'CALCUL.INC'
	include 'IF.INC'

C ***
C ***	Functions
C ***
	real pol, neweta

C ***
C ***	Local variables
C ***
	logical extra, syn, xi_fit
	integer i, m
	real value( nfit ), delta, old_value, ignore


C ***	Increments for partial derivatives:

	real delta_abu, delta_vsini, delta_xi_macro, delta_xi_micro
	parameter( delta_abu = 0.1, delta_vsini = 0.1,
     >             delta_xi_macro = 0.1,
     >             delta_xi_micro = 0.1 )


	if( der_calc ) then
C ***
C ***	   compute partial derivatives
C ***
C ***	   assumes YFIT has values at current LAMOBS positions
C ***
	   do m = 1, nterms
              value( m ) = a( m )
	   end do
	   do m = 1, nterms
	      if( fit_item(m) .eq. 14 ) then
C ***
C ***	         v sini; must be done before abundances because it uses
C ***	                 existing RSCAN.
C ***
	         delta = delta_vsini*value( m )
		 old_value = vsini
	 	 vsini = value( m ) + delta
	         call falt( boa(1) )
                 vsini = old_value
	         do i = 1, mobs
	            deriv( i, m ) = ( pol( lamobs(i), lascan, rstreu, 1,
     >                              mma, 1, extra ) - yfit(i) )/delta
	         end do
	         if( boa(1) ) call plotout( 3 )
	      else if( fit_item(m) .eq. 13 ) then
C ***
C ***	         xi macro; must be done before abundances because it uses
C ***	                   existing RSCAN.
C ***
	         delta = delta_xi_macro*value( m )
		 old_value = ximac
	 	 ximac = value( m ) + delta
	         call falt( boa(1) )
                 ximac = old_value
	         do i = 1, mobs
	            deriv( i, m ) = ( pol( lamobs(i), lascan, rstreu, 1,
     >                              mma, 1, extra ) - yfit(i) )/delta
	         end do
	         if( boa(1) ) call plotout( 3 )
	      end if
	   end do
	   do m = 1, nterms
	      if( fit_item(m) .ge. 98 ) then
C ***
C ***	         Abundance
C ***
	         delta = delta_abu
	         old_value = value( m )
	         value( m ) = value( m ) + delta
	         ignore = neweta( value )
	         call synthesis( boa(1) )
	         value( m ) = old_value
	         do i = 1, mobs
	            deriv( i, m ) = ( pol( lamobs(i), lascan, rstreu, 1,
     >                              mma, 1, extra ) - yfit(i) )/delta
	         end do
	         if( boa(1) ) call plotout( 3 )
              end if
 	   end do
	   do m = 1, nterms
	      if( fit_item(m) .eq. 55 ) then
C ***
C ***	         microturbulence
C ***
	         delta = delta_xi_micro*value( m )
	         old_value = value( m )
	         value( m ) = value( m ) + delta

	         call newmicro( value )

                 call synthesis( boa(1) )

	         value( m ) = old_value
	         call newmicro( value )

	         do i = 1, mobs
	            deriv( i, m ) = ( pol( lamobs(i), lascan, rstreu, 1,
     >                              mma, 1, extra ) - yfit(i) )/delta
	         end do

	         if( boa(1) ) call plotout( 3 )

	      end if
 	   end do
	else
C ***
C ***	   New spectrum
C ***
	   xi_fit = .false.
	   syn = .false.
	   do m = 1, nterms
	      if( fit_item(m) .eq. 14 ) then
	         vsini = a( m )
                 syn = .false. .or. syn

	      else if( fit_item(m) .eq. 13 ) then
	         ximac = a( m )
                 syn = .false. .or. syn

              else if( fit_item(m) .eq. 55 ) then
                 syn = .true.
                 xi_fit = .true.

	         call newmicro( a )

	      else
                 syn = .true.

	      end if

	   end do
	   if( syn ) then
              if( .not.xi_fit ) ignore = neweta( a )
	      call synthesis( boa(1) )

	   else

              call falt( boa(1) )
	   end if

	   do i = 1, mobs
	      yfit( i ) = pol( lamobs(i), lascan, rstreu, 1, mma, 1,
     >                         extra )
	   end do

	end if
	end
