     	function linop( depth, k, lam, dlam )
C ***
C ******************************************************************************
C ***
C ***	Returns line opacity.
C ***
C ***	depth	i >	Depth point for which profile required
C ***	k	i >	number of line
C ***	lam	r >	wavelength []
C ***	dlam	r >	wavelength [] from line center
C ***
C ***	23-FEB-1994 15:14:26.00		ML / IoA
C ***    5-APR-1995 16:06:23.00         ML / IoA.  He broadening added 
C ***	30-APR-1997 16:49:47.48		ML / Bamberg.  HEBCSS -> HE1
C ***	14-SEP-1997 17:48:25.00		ML / Stegaurach.  HY(K) -> LPROF
C ***	22-OCT-1997 15:11:12.56		ML / Bamberg.  He I Beauchamp et al.
C ***	31-OCT-1997 18:06:38.83		ML / Bamberg.  VCSSB.
C ***
C ******************************************************************************
C ***
	implicit none

C ***
C ***	Arguments
C ***
	integer depth, k
	real lam, dlam, linop

C ***
C ***	Global variables
C ***
	include 'LINLTE.INC'
	include 'LINE.INC'
	include 'LINDAT.INC'
	include 'IONS.INC'
	include 'STRUCT.INC'
	include 'PRES.INC'

C ***
C ***	Local variables
C ***
	real x

C ***
C ***	Functions
C ***
	integer he1
	real profhy, profvcs, profvcssb, profhe, h, xp, hlinop

	xp( x ) = exp( 2.30258*x )

	if( lprof(k) .eq. PVOIGT ) then
           linop = h( alpha(depth, k),
     >                abs(dlam)/lamk(k)/vdop(depth, kj(k)) )*
     >                eta0( depth, k )
	else if( lprof(k) .eq. PVCS ) then
           linop = profvcs( abs(dlam), depth, k )*eta0( depth, k )

	else if( lprof(k) .eq. PVCSSB ) then
           linop = profvcssb( abs(dlam), depth, k )*eta0( depth, k )

	else if( lprof(k) .eq. PHEGRIEM .or.
     >           lprof(k) .eq. PHEBCSS .or.
     >           lprof(k) .eq. PHEBWB ) then
           linop = profhe( lam, dlam, depth, k )*eta0( depth, k )

	else if( lprof(k) .eq. PHY ) then
           linop = profhy( alpha(depth, k), abs(dlam), depth, k )*
     >             eta0( depth, k )

	else if( lprof(k) .eq. PHOP ) then
           linop = hlinop( lam, 5040./theta(depth), xp(pelg(depth)) )*
     >             xp( zetaj(depth,1) )/roptot( depth )

	else	! programming error.
	   write( stdout, * ) 'LINLTE>> Error: Unknown profile, line',
     >                        k, lprof(k)
	   call exit
        end if
         
	end

