        subroutine matinv( array, norder, det )
C ***
C ******************************************************************************
C ***
C ***	Routine taken from: Philip R. Bevington, `Data Reduction and Error 
C ***                       Analysis for the Physical Sciences'
C ***                       McGraw-Hill, New York, 1969, pp. 302-303
C ***
C ***	Purpose:
C ***	  Invert a symmetric matrix and calculate its determinant
C ***
C ***	Description of parameters
C ***     ARRAY   ><  Input matrix which is replaced by its determinant
C ***	  NORDER   >  Degree of matrix (order of determinant)
C ***	  DET	   <  Determinant of input matrix
C ***
C ***
C ***	Modifications:
C ***	  23-APR-1992 23:29:13.55	ML / UT Austin.  Fixed bug.
C ***	   4-JUN-1992 23:31:40.15       ML / UT Austin.  Adapted to LINLTE
C ***
C ***
C ***	19-APR-1992 07:27:02.53
C ***
C ***
C ******************************************************************************
C ***
	implicit none

	include 'LINLTE.INC'


C ***
C ***	Arguments
C ***

        integer norder
        real array( nfit, nfit ), det


C ***
C ***	Local variables
C ***

        integer ik( nfit ), jk( nfit ), i, j, k
	real amax, save

        det = 1.
        do k = 1, norder
C ***
C ***      Find largest element ARRAY( I, j ) in rest of matrix
C ***
           amax = 0.
1          continue
           do i = k, norder
              do j = k, norder
                 if( abs(amax) .le. abs(array(i, j)) ) then
                    amax = array( i, j )
                    ik( k ) = i
                    jk( k ) = j
                 end if
              end do
           end do
C ***
C ***      Interchange rows and columns to put AMAX in ARRAY( K, K )
C ***
           if( amax .eq. 0. ) then
              det = 0.
              return

           end if
           i = ik( k )
           if( i .lt. k ) go to 1
           if( i .gt. k ) then
              do j = 1, norder
                 save = array( k, j )
                 array( k, j ) = array( i, j )
                 array( i, j ) = -save
              end do
           end if
           j = jk( k )
           if( j .lt. k ) go to 1
           if( j .gt. k ) then
              do i = 1, norder
                 save = array( i, k )
                 array( i, k ) = array( i, j )
                 array( i, j ) = -save
              end do
           end if
C ***
C ***      accumulate elements of inverse matrix
C ***
           do i = 1, k - 1
              array( i, k ) = -array( i, k )/amax
           end do
           do i = k + 1, norder
              array( i, k ) = -array( i, k )/amax
           end do
           do i = k + 1, norder
              do j = 1, k - 1
                 array( i, j ) = array( i, j ) +
     >                           array( i, k )*array( k, j )
              end do
              do j = k + 1, norder
                 array( i, j ) = array( i, j ) +
     >                           array( i, k )*array( k, j )
              end do
           end do
           do i = 1, k - 1
              do j = 1, k - 1
                 array( i, j ) = array( i, j ) +
     >                           array( i, k )*array( k, j )
              end do
              do j = k + 1, norder
                 array( i, j ) = array( i, j ) +
     >                           array( i, k )*array( k, j )
              end do
           end do
           do j = 1, k - 1
              array( k, j ) = array( k, j )/amax
           end do
           do j = k + 1, norder
              array( k, j ) = array( k, j )/amax
           end do
           array( k, k ) = 1./amax
           det = det*amax
        end do
C ***
C ***   Restore ordering of matrix
C ***
        do k = norder, 1, -1
           j = ik( k )
           if( j .gt. k ) then
              do i = 1, norder
                 save = array( i, k )
                 array( i, k ) = -array( i, j )
                 array( i, j ) = save
              end do
           end if
           i = jk( k )
           if( i .gt. k ) then
              do j = 1, norder
                 save = array( k, j )
                 array( k, j ) = -array( i, j )
                 array( i, j ) = save
              end do
           end if
        end do

        end
