	function profhe( lam, dlam, depth, k )
C ***
C ******************************************************************************
C ***
C ***	Returns Helium profile from BCSS theory.
C ***
C ***	lam	>  R	Current wavelength []
C ***	dlam	>  R	distance from line center []
C ***	depth	>  I	depth point of interest
C ***	k  	>  I	number of blend line
C ***
C ***	Modifications:
C ***
C ***	 5-APR-1995 16:14:56.00		ML / IoA Cambridge.  Created.
C ***	29-APR-1997 11:31:58.00		ML / Bamberg.  Griem stuff added.
C ***                                   Calculation of Doppler width corrected
C ***                                   -- used lam instead of lamk(k)
C ***   14-SEP-1997 17:48:25.00		ML / Stegaurach.  PHEBCSS... stuff.
C ***	22-OCT-1997 15:24:12.84		ML / Bamberg.  Beauchamp et al.
C ***   28-OCT-1997 22:00:12.11         ML / Stegaurach.  Add some variables
C ***                                   to BWB call.
C ***	 5-DEC-2003 01:03:45.55		ML / Stegaurach.  Bug fix in error
C ***					message (harmless)
C ***
C ******************************************************************************
C ***
	implicit none

C ***
C ***	Arguments
C ***
        integer depth, k
	real lam, dlam, profhe

C ***
C ***	Global variables
C ***
        include 'LINLTE.INC'
	include 'STRUCT.INC'
	include 'PRES.INC'
	include 'LINE.INC'
	include 'LINDAT.INC'

C ***
C ***	Local variables
C ***
	integer iline

	real kboltz, c_light, pi
	parameter( kboltz = 1.38062e-16, c_light = 2.997925e18,
     >             pi = 3.14159265358979323846264 )

	real temp, ne
	real x

C ***
C ***	Functions
C ***
	integer hebcss, hegriem, hebwb
        real bwb, bcss, linehe1, exp10

	exp10( x ) = exp( 2.302585093*x )

C ***
C ***	Code
C ***

	temp = 5040./theta( depth )
	ne = exp10( pelg(depth) )*theta(depth)/5040./kboltz

	if( lprof(k) .eq. PHEBWB ) then
C ***
C ***	   BWB
C ***
	   iline = hebwb( k )
	   if( iline .le. 0 ) then
	       write( stdout, * ) 'LINLTE>> No BWB theory available ',
     >                            'for line #', k
	       write( stdout, * ) '         element', namk(k), ' at ',
     >                             lamk(k), ''
               write( stdout, * ) '         Modify line data in '//
     >                            'LINE.DAT, set C4LG .ne. -4'
	       call exit
	   end if
	   profhe = bwb( temp, ne, iline, lam, dlam,
     >                   vdop(depth,kj(k))*lamk(k), k )
	else if( lprof(k) .eq. PHEBCSS ) then
C ***
C ***	   BCSS
C ***
	   iline = hebcss( k )
	   if( iline .le. 0 ) then
	       write( stdout, * ) 'LINLTE>> No BCSS theory available ',
     >                            'for line #', k
	       write( stdout, * ) '         element', namk(k), ' at ',
     >                             lamk(k), ''
               write( stdout, * ) '         Modify line data in '//
     >                            'LINE.DAT, set C4LG .ne. -3'
	       call exit
	   end if
	   profhe = bcss( depth, temp, ne, iline, lam, dlam,
     >                    vdop(depth,kj(k))*lamk(k) )
	else if( lprof(k) .eq. PHEGRIEM ) then
C ***
C ***	   Use Griem tables.
C ***
	   iline = hegriem( k )
	   if( iline .le. 0 ) then
	       write( stdout, * ) 'LINLTE>> No Griem data available ',
     >                            'for line #', k
	       write( stdout, * ) '         element', namk(k), ' at ',
     >                             lamk(k), ''
               write( stdout, * ) '         Modify line data in '//
     >                            'LINE.DAT, set C4LG .ne. 0'
	       call exit
	   end if
	   profhe = linehe1( temp, ne, iline, lamk(k), dlam,
     >                       vdop(depth,kj(k))*lamk(k) )
	else
C ***
C ***	   Programming error -- function should not have been called
C ***
	   write( stdout, * ) 'PROFHE>>  HE1 returned invalid result ',
     >                        lprof(k), 'for line #', k, ', ', lamk(k),
     >                        ''
	   call exit
	end if
	end
