	subroutine scross( xmin, xmax, freq0, star0, fi0, nfp, shift )
C ***
C ******************************************************************************
C ***
C ***	Perform cross correlation between arrays star0 and fi0.
C ***
C ***	xmin	r >	x value [A] at which to start x correlation
C ***	xmax	r >	x value [A] at which to stop x correlation
C ***	freq0	r >	wavlengths [A]
C ***	star0	r ><	first array to correlate, also used as workspace
C ***	fi0	r ><	second array to correlate, also used as workspace
C ***	nfp	i >	number of points in freq0, star0, fi0
C ***	shift	r <	shift [A] between the two arrays
C ***
C ***	 4-JUL-1996 19:10:43.65	     ML / Bamberg.  Made from Figaro routines
C ***	24-JUL-1996 19:40:50.91	     ML / Bamberg.  Taken from FITPROF
C ***   27-MAR-1997 01:11:09.00      ML / Stegaurach.  MX moved to LINLTE.INC
C ***   28-APR-1997 12:32:11.00      ML / Stegaurach.  Check for MX correctness.
C ***	16-JAN-1998 19:58:19.18	     ML / Bamberg.  Check number of points for
C ***				     doing cross correlation.  Punt if < 8.
C ***
C ******************************************************************************
C ***
	implicit none

C ***
C ***	Arguments
C ***
	real xmin, xmax
	integer nfp
	real freq0(nfp), star0(nfp), fi0(nfp)
	real shift

	include 'LINLTE.INC'

C ***
C ***	Local variables
C ***
!        integer mx
!        parameter( mx = 2**int(log(real(nobs))/log(2.) + 1) )
!			! let's see if our Fortran does this

	integer kz(4)
	complex ftstar(mx), ftobs(mx), ftcross(mx)	! Fourier transforms
	real xv(mx)
	real cross(mx)	! correlation function
	real cosbell_frac
	real shi, width

	integer start, end, i, nx0, nx2
	logical cont_fit

	real gen_power2
	external gen_power2, locate

C ***
C ***   This if block can be taken out if expression in parameter statement
C ***   could be used.
C ***
        if( mx .lt. 2**int(log(real(nobs))/log(2.) + 1) ) then
           write( stdout, * ) 'LINLTE>> ERROR: Dimension error '//
     >                        'for MX.  See LINLTE.INC for fix.'
           call exit
        else if( mx .gt. 2**int(log(real(nobs))/log(2.) + 1) ) then
           write( stdout, * ) 'LINLTE>> WARNING: MX set too big in'//
     >                        'LINLTE.INC.  Wasting some memory.'
        end if



	call locate( freq0, nfp, xmin, start )
	call locate( freq0, nfp, xmax, end )


	nx0 = end - start + 1
	if( nx0 .lt. 8 ) then
	   write( stdout, * ) 'LINLTE>> Warning: Too few points for '//
     >                        'finding radial velocity.'
           shift = 0.
	   go to 99	! return
	end if

	i = gen_power2( nx0, nx2 )	! get an integral power of 2

	kz(1) = 5
	kz(2) = min( 20, nx2 - 2 )
	kz(3) = max( nx2/6, kz(2) + 1 )
	kz(4) = min( 2*kz(3), nx2 )

	cosbell_frac = 10.
	cont_fit = .true.

	call fig_cross( star0(start), fi0(start), nx0, nx2, cont_fit,
     >                  cosbell_frac, kz, .true., ftstar, ftobs,
     >                  ftcross, xv, cross, shi, width )


	shift = (freq0(end) - freq0(start))*shi/(nx0-1)

99	continue
	end
