	subroutine spefit
C ***
C ******************************************************************************
C ***
C ***	Fits synthetic spectrum to observation using Levenberg-Marquardt
C ***	algorithm.
C ***
C ***	25-MAY-1992 02:13:19.11		ML / UT Austin
C ***
C ***	Modifications:
C ***
C ***	 3-JUN-1994 19:48:07.00		ML / IoA Cambridge.  Added abort option
C ***	22-SEP-1994 19:26:42.00		ML / IoA.  Removed arguments. Done now
C ***					via common/obs/
C ***					Put fit_value, start_value, chisqr in
C ***                                   common/fit/
C ***	23-SEP-1994 19:38:57.00		ML / IoA.  Changed treatment of
C ***					LIMITS.DAT to deal with multiple
C ***                                   syntheses in LINE.DAT.
C ***	28-SEP-1994 15:36:32.00		ML / IoA.  Changed logic, added resize,
C ***					repeat fit option
C ***	23-NOV-1994 21:09:52.00		ML / IoA.  Fixed writing of LIMITS.DAT
C ***	24-JUL-1996 19:03:27.05		ML / Bamberg.  RV correction...
C ***	18-DEC-1997 16:26:34.65		ML / Bamberg.  Add simple comment
C ***					option for LIMITS.DAT:  -1 -1 comment
C ***                                   Not written out, though.
C ***	18-DEC-1997 17:17:56.64		ML / Bamberg.  Limits from both file
C ***                                   and cursor.
C ***	16-JAN-1998 17:12:54.15		ML / Bamberg.  Moved RV correction
C ***					further up.
C ***	10-FEB-1998 14:53:18.98		ML / Bamberg.  Move it back. 
C ***					Was utterly wrong.
C ***    1-DEC-2003 23:27:00.00         ML / Stegaurach.   Calculate fit errors.
C ***
C ******************************************************************************
C ***
	implicit none

C ***
C ***	Global variables
C ***
	include 'LINLTE.INC'
	include 'CALCUL.INC'
	include 'PROFIL.INC'
	include 'IF.INC'
	include 'ELEM.INC'
	include 'LINE.INC'
	include 'PRES.INC'
	include 'PRINT.INC'
	include 'RECT_SIZE.INC'
	include 'OBS.INC'
	include 'FIT.INC'
	include 'MON_CODES.INC'
	include 'ERASURE.INC'

C ***
C ***	Functions
C ***
	external linfit, get_limits
	integer mon
	real pol

C ***
C ***	Local variables
C ***

	logical graph, extra

	real clight
	parameter( clight = 2.99793e5 )

	integer i, j, k, l, itmax
	parameter( itmax = 5 )
	double precision old_chisqr
	real flambda, h, x1, x2

	integer ist
	include 'FILES.INC'

	real z, width


	do i = 1, mfit
C ***
C ***	   initialize start values
C ***
           if( fit_item(i) .ge. 98 ) then
C ***      
C ***	      Abundance (relative to ATM.DAT/LINE.DAT flag 53)
C ***      
              fit_value( i ) = 0.

              j = fit_item( i )/100
	      k = fit_item( i ) - 100*j
              do l = 1, ima
                 if( j .eq. nami(l)/100 ) then
                    start_value( i ) = abui( l ) + dloggf( j, k )
                    go to 1
              
	         end if
              end do
1             continue
              
           else if( fit_item(i) .eq. 14 ) then
C ***      
C ***         vsini
C ***      
              fit_value( i ) = vsini

           else if( fit_item(i) .eq. 13 ) then
C ***      
C ***         macro trubulence
C ***      
              fit_value( i ) = ximac

           else if( fit_item(i) .eq. 55 ) then
C ***      
C ***         microturbulence
C ***      
              fit_value( i ) = xi( 2 )
	   else
C ***
C ***	      should never happen, has been checked in ANALYZE;
C ***                              just for catching programming errors.
C ***
	      stop 'SPEFIT internal error'
           end if
        end do
	call eta_init( fit_value )

C ***
C ***	Find border regions
C ***
	z = vsini/clight
	width = 3.5*0.5*(endlam + anflam)/clight*ximac*0.60056
	
	cbmin = int( 1000.*width/dellam ) + 1
	cbmax = mma - cbmin
        
	cbmin = max( cbmin,
     >               int( 1000.*(lascan(1)*z/(1 - z))/dellam ) + 2 )
	cbmax = min( cbmax,
     >               int( 1000.*(lascan(mma)/(1 + z) - lascan(1))/dellam
     >                    + 1 )
     >             )
            
C ***
C ***	Compute starting solution
C ***
        do j = 1, mobs
           yfit(j) = pol( lamobs(j), lascan, rstreu, 1, mma, 1, extra )
        end do

	call rvcorr	! Radial velocity correction (needs cbmin/cbmax set)

	do i = 1, mobs         
	   if( lamobs(i) .ge. lascan(cbmin) ) then
	      cbmin = i
              go to 2
	   end if
	end do
2	continue

	do i = mobs, cbmin, -1
	   if( lamobs(i) .lt. lascan(cbmax) ) then
	      cbmax = i
              go to 3
	   end if
	end do
3	continue
	
	graph = boa(1) .or. itv .gt. 1 .or. iflimits .eq. 1 .or.
     >          iflimits .eq. 3


10	continue	! start point for refit of same spectra/line data

C ***
C ***	Chop off border regions
C ***
        do i = 1, cbmin
           fit_mask(i) = 0.
        end do

	do i = cbmin+1, cbmax-1
	   fit_mask(i) = 1.
	end do

        do i = cbmax, mobs
           fit_mask(i) = 0.
        end do

	xmin = xpmin
	xmax = xpmax
	ymin = ypmin
	ymax = ypmax
	call plotout( 1 )   ! frame with obs + syn
	if( pid ) call plotout(8)	! line id's
        

	call plotout( 2 )  ! plot fit_mask, here that is only border region

C ***
C ***	Select region to fit (cursor/file)
C ***
	if( iflimits .eq. 1 ) then
C ***
C ***	   cursor
C ***
	   call resize( get_limits, ist )
	   if( ist .eq. end_fit ) go to 4	! abort
	   call limout				! fit ranges to LIMITS.DAT

     	else if( iflimits .eq. 2 .or. iflimits .eq. 3 ) then
C ***
C ***	   from file and possibly cursor
C ***
C ***	   open LIMITS.DAT, status = old; done in ANALYZE.

	   read( LIMITS_DAT, *, iostat = ist ) x1, x2
	   do while( ist .eq. 0 .and. x1 .ne. -5 .and. x2 .ne. -5 )
	      if( x1 .ne. -1. ) then
                 h = x1
                 x1 = min( x1, x2 )
                 x2 = max( h, x2 )
                 j = 1
                 do while( lamobs(j) .lt. x1 .and. j .lt. mobs )
                    j = j + 1
                 end do
                 k = 1
                 do while( lamobs(k) .lt. x2 .and. k .lt. mobs )
                    k = k + 1
                 end do
                 do j = j, k
                    fit_mask(j) = 0.
                 end do
              end if
	      read( LIMITS_DAT, *, iostat = ist ) x1, x2
	   end do
	   call plotout( 2 )	! plot fit_mask

	   if( iflimits .eq. 3 ) then           ! cursor selection
	      call resize( get_limits, ist )
	      if( ist .eq. end_fit ) go to 4	! abort
	      call limout			! fit ranges to LIMITS.DAT
           end if
	end if

	call putout( 6 )	! print fit ranges

C ***
C ***	Do the fit
C ***
	if( graph ) call tplane( Erase, Single_plane )
	if( iflimits .eq. 1 ) then
	   write( stdout, * ) 'Starting fit...'
	end if
	chisqr = 0.d0
	do i = 1, mobs
	   chisqr = chisqr +
     >              ( dble( fit_mask(i)*(fluxobs(i) - yfit(i)) ) )**2
	end do
	chisqr = chisqr/(mobs - mfit)
	old_chisqr = 0.d0

	flambda = 0.001
	ist = cont_fit
	it = 0
!	do while( (ist .eq. cont_fit) .or. 
!     >            (ifauto .eq. 1 .and.
!     >             (old_chisqr - chisqr)/chisqr .ge. 0.1d0 .and.
!     >             it .le. itmax
!     >            ) )
	do while( (ist .eq. cont_fit) .or. 
     >             it .lt. ifauto )

C ***
C ***	   main fitting loop
C ***

	   if( boa(1) ) call putout( 3 )

	   it = it + 1
           old_chisqr = chisqr

	   call curfit( lamobs, fluxobs, mobs, mfit, fit_value, flambda,
     >       yfit, chisqr, linfit, fit_mask, fit_err )

	   call plotout( 3 ) 	! overplot new fit

	   call putout( 7 )

	   if( chisqr .ge. old_chisqr ) then
	      write( stdout, * ) char(7), 'No Improvement!'
	      write( 3, '(1x, a)' ) 'No Improvement!'
	      ip = ip + 1
	   end if

	   if( ifauto .eq. 0 ) then
              ist = mon()
	   else
              ist = end_fit
	   end if

	end do

4	continue		! for abort
        call plotout( 7 )	! write fit results in plot
	call plotout( 4 )	! finish plot
C ***
C ***	create output
C ***
	call falt( .not.boa(1) )

	if( ist .eq. again_fit ) go to 10

	end
