        function sumeta( n, dl, sumet2 )
C ***
C ******************************************************************************
C ***
C ***	Calculates total continuous or line opacity depending on CONTI.
C ***
C ***	Arguments:
C ***
C ***	   N		> I	Depth point for which to return opacity.
C ***	   DL		> R	Wavelength [m] at which to return opacity.
C ***	   SUMET2	< R	Help variable to reconstruct opacity without
C ***				absorption terms in case of NLTE.
C ***
C ***	Remarks:
C ***
C ***	   For syntheses the total opacity is calculated in the first call for
C ***	   each wavelength point of the synthesis interval.  Subsequent calls
C ***	   return only the values stored in OPAC and OPAC2.  The initialization
C ***	   is controled by OPINIT.
C ***
C ***	   This allows to compute line opacities only where it is needed
C ***	   starting at the line center and proceeding towards both wings.
C ***	   Calculation stops when the line opacity reached
C ***      1.e-5/<number of lines> of the total opacity summed up so far at
C ***	   a given wavelength point, minimizing the number of Voigt function
C ***	   evaluations.  Note that this gives accurate opacities *only* at
C ***	   exactly the wavelength points of the synthesis interval and not
C ***	   at arbitrary wavelengths.
C ***
C ***	   A second entry NEWETA allows to update ETA0 and related quantities
C ***	   for a different set of abundances with only minmal recomputation.
C ***	   This is used for matching spectra to observations, i.e.,
C ***	   if BOA(4) .AND. BOA(2) .eq. .TRUE..
C ***
C ***
C ***	23-JUN-1992 20:58:08.21
C ***
C ***	Modifications:
C ***
C ***	24-JAN-1993 22:51:27.38		ML / UT Austin.  Changed EPSILON from
C ***					1e-4 to 1e-5.
C ***
C ***	23-FEB-1994 18:13:36.00		ML / IoA Cambs.  Added H line opacity,
C ***					some remodeling
C ***
C ***    2-MAY-1994 13:14:11.00		ML / IoA Cambs.  Fixed call to linop
C ***					for non-synthesis branch
C ***
C ***	31-MAY-1996 19:42:59.35		ML / Bamberg.  Added IMPORT to remember
C ***                                   whether line has been added in opac.
C ***                                   So we can plot them later.
C ***
C ***	 4-MAR-1997 20:08:28.02		ML / Bamberg.  Removed abs from dlam.
C ***                                   Needed for asymetric He profiles.
C ***
C ***   16-APR-1997 23:13:06.00         ML / Stegaurach.  Save varepsilon
C ***   15-JUN-1997 16:44:56.00         ML / Stegaurach.  Save opac, opac2.
C ***                                   Don't know if this caused a problem.
C ***	16-JUN-1997 13:00:44.00		ML / Bamberg.  Swapped indices in
C ***					opac and opac2.  Makes code better
C ***					swappable.
C ***
C ******************************************************************************
C ***
	implicit none

C ***
C ***	Arguments
C ***
	integer n
	real sumeta, sumet2, dl

C ***	Arguments ENTRY NEWETA
	real value(*), neweta

C ***
C ***	Global variables
C ***
	include 'LINLTE.INC'
	include 'IF.INC'
      	include 'FLUSS.INC'
      	include 'LINE.INC'
      	include 'LINDAT.INC'
      	include 'STRUCT.INC'
	include 'CALCUL.INC'
	include 'PROFIL.INC'

C ***
C ***	Functions
C ***
	real linop

C ***
C ***	Local variables
C ***
	logical update, do_this_line
	integer i, k, l, m, ls, nd
	real dlam, bnlte, cnlte, op, opa
	real epsilon, varepsilon, ratio, plim   !, macheps
*	real opac( ndepth, ipkt ), opac2( ndepth, ipkt )
	real opac( ipkt, ndepth ), opac2( ipkt, ndepth )
	real diff( nfit ), exp_diff( nfit )
	parameter( epsilon = 1e-5, plim = 0.1 )!, macheps = 5.e-7 )
                               !   machine epsilon so that 1. + macheps .ne. 1.
        save varepsilon, opac, opac2


	update = .false.
        sumeta = etac( n )
        sumet2 = etac( n )

	go to 1


	entry neweta( value )
C ***         ======

	update = .true.
	opinit = .true.

1	continue
        if( .not. conti ) then
	   if( .not. boa(4) ) then
              do k = 1, kma
!                 dlam = abs( dl - dlk(k) )/1000.
                 dlam = (dl - dlk(k))/1000.
	         op = linop( n, k, (dl + 1000.*lamk(1))/1000., dlam )
	         call bfac( n, k, cnlte, bnlte )
                 sumet2 = sumet2 + op*bnlte
                 sumeta = sumeta + op*cnlte
              end do
	   else
              if( opinit ) then
                 opinit = .false.
                 if( .not.update ) then
*                    do l = 1, iend
*                       do nd = 1, nma
*                          opac( nd, l ) = 0.
*                          opac2( nd, l ) = 0.
*	               end do
*                    end do
                    do nd = 1, nma
                       do l = 1, iend
                          opac( l, nd ) = 0.
                          opac2( l, nd ) = 0.
	               end do
                    end do
                    varepsilon = epsilon/kma
                 else
C ***
C ***               compute update factors to ETA0 and log gf
C ***
	            call deleta( value, diff, exp_diff )

	         end if
                 do l = 1, iend
                    calflu( l ) = .false.
                 end do
                 do k = 1, kma
                    if( .not.update ) then
                       do_this_line = .true.
                    else
C ***
C ***                  check if line has changed
C ***
                       do_this_line = .false.
                       if( .not.blend(k) ) then
                          do m = 1, mfit
                             if( namk(k) .eq. fit_item(m) .and.
     >                           diff(m) .ne. 0. ) then
C ***
C ***	                        if so, compute new log gf and ETA0.
C ***
                                do_this_line = .true.
                                gflg( k ) = gflg( k ) + diff( m )
                                do i = 1, nma
                                   eta0( i, k ) = eta0( i, k )*
     >                                            exp_diff( m )
                                end do
                                go to 2
                             
                             end if
                          end do
	               end if
                    end if
2                   continue
	            if( do_this_line ) then
                       ls = dlk(k)/dellam + 1
	               ls = max( 1, min( ls, iend ) )
                       import(k) = .false.
                       do nd = 1, nma
	                  call bfac( nd, k, cnlte, bnlte )
                          l = ls
                          do i = 1, -1, -2
C ***                  
C ***                        wavelength loop;
C ***
C ***                        start at line center, move up;
C ***                        second pass move down
C ***                  
                             do while( l .ge. 1 .and. l .le. iend )
C ***                  
C ***                           add line opacity to total
C ***
!                                dlam = abs( 0.001*(1000.*lascan(l) - 
                                dlam =    ( 0.001*(1000.*lascan(l) - 
     >                                             1000.*lamk(k)) )
	                        op = linop( nd, k, lascan(l), dlam )
*                                ratio = op*cnlte/(etac(nd) + opac(nd,l))
                                ratio = op*cnlte/(etac(nd) + opac(l,nd))
                                import(k) = ratio .gt. plim
     >                                      .or. import(k) ! depth loop!
	                        if( ratio .le. varepsilon ) go to 3

C ***                                                 Break

C ***
C ***                           is important, so add it
C ***
!d	if( .not.calflu(l) ) type *, '.true. ', nd, lascan(l),
!d     >        op*cnlte/(etac(nd) + opac(nd,l)), op, etac(nd),
!d     >        opac(nd,l)
                                if( .not.update ) then
	                           opa = op
	                        else
	                           opa = op*( 1. - 1./exp_diff( m ) )
                                end if
*	                        opac( nd, l ) = opac( nd, l ) +
	                        opac( l, nd ) = opac( l, nd ) +
     >                                          opa*cnlte
*	                        opac2( nd, l ) = opac2( nd, l ) +
	                        opac2( l, nd ) = opac2( l, nd ) +
     >                                           opa*bnlte
                                calflu( l ) = .true.

                                l = l + i

                             end do   ! while

3                            continue
                             do while( update .and.
     >                                 l .ge. 1 .and. l .le. iend )
C ***                  
C ***                           Subtract old line opacity from total
C ***                  
!                                dlam = abs( 0.001*(1000.*lascan(l) - 
                                dlam =    ( 0.001*(1000.*lascan(l) - 
     >                                             1000.*lamk(k)) )
	                        op = linop( nd, k, lascan(l), dlam )/
     >                               exp_diff( m )
*	                        if( op*cnlte/(etac(nd) + opac(nd,l))
	                        if( op*cnlte/(etac(nd) + opac(l,nd))
     >                              .le. varepsilon ) go to 4

C ***                                                 Break

C ***
C ***                           was important, so remove it
C ***
*	                        opac( nd, l ) = opac( nd, l ) -
	                        opac( l, nd ) = opac( l, nd ) -
     >                                          op*cnlte
*	                        opac2( nd, l ) = opac2( nd, l ) -
	                        opac2( l, nd ) = opac2( l, nd ) -
     >                                           op*bnlte
                                calflu( l ) = .true.

                                l = l + i

                             end do   ! while

4                            continue

                             l = ls - 1
                          end do   ! i

	               end do   ! nd

	            end if   ! do_this_line
	         end do   ! kma

              end if   ! opinit
              if( .not.update ) then
                  l = (dl/dellam + 0.5) + 1
                  l = min( max( 1, l ), iend )
*                  sumeta = sumeta + opac( n, l )
                  sumeta = sumeta + opac( l, n )
*                  sumet2 = sumet2 + opac2( n, l )
                  sumet2 = sumet2 + opac2( l, n )
	      end if
          end if   ! boa(4)
        end if   ! conti

        end
