	function vcssb( dlam, t, ne, kma )
C ***
C ******************************************************************************
C ***
C ***	Interpolate VCSSB tables for He II.
C ***
C ***	DLAM	>  R	delta lambda from line center []
C ***	T	>  R	temperature [K]
C ***	ne	>  R	electron density [cm^-1]
C ***	kma	>  I	number of blend line
C ***
C ***	Expects lower and upper quantum numbers in LU and DIU from
C ***	COMMON /LINDAT/.
C ***
C ***	Modifications:
C ***
C ***	31-OCT-1997 18:30:29.66		ML / Bamberg.  Made from VCS.FOR
C ***
C ******************************************************************************
C ***
	implicit none

C ***
C ***	Arguments
C ***
        real dlam, t, ne, vcssb
	integer kma

C ***
C ***	Global variables
C ***
	include 'LINLTE.INC'
	include 'FILES.INC'
	include 'LINDAT.INC'

C ***
C ***	Local variables
C ***
	integer pmt, pmne, pmp, mline, mij
	parameter( pmt = 6,	! # of temp points in VCSSB table
     >             pmne = 11,   ! # of density points in VCSSB table
     >             pmp = 50,    ! # of profile points in VCSSB table
     >             mline = 20,  ! # of He lines in VCSSB table
     >             mij = 17     ! max principal quantum number of VCSSB table
     >           )

	integer ij(mij,mij)		    ! x-ref to identify line
	integer mp(mline), mne(mline), mtemp(mline)
     	real log_alpha0(mline), log_t0(mline),
     >       log_alpha_inc(mline), log_t_inc(mline),
     >       log_nel(pmne,mline),
     >       ne0

        real svcs(pmt, pmne, pmp, mline)	! actual VCSSB table
	real prof(pmp)				! T/ne interpolated VCSSB profile

	real f0, dl

	integer i, j, k, m, line, nl(mline), nu(mline), nnl, nnu

	integer old_line
	real old_t, old_ne 
	data old_t /-1./, old_ne /-1./, old_line /-1/

	integer itemp, ine, ialp
	real log_t, log_ne, log_alpha, btemp, bne, balp
	real wttemp, wtne, wt

	real x

	logical first 
	data first /.true./

C ***
C ***	Functions
C ***
	real exp10

	save first, old_t, old_ne, old_line, prof, svcs, ij
	save log_alpha0, log_t0,
     >       log_alpha_inc, log_t_inc, log_nel,
     >       mp, mne, mtemp
        save ne0

        exp10( x ) = exp( 2.30258509299405E0*x )

C ***
C ***	Code
C ***
	if( first ) then
	   first = .false.
!	   open( vcssb_dat, file = 'VCSSB.DAT', status = 'OLD' )
	   do i = 1, mij
	      do j = 1, mij
	         ij( j, i ) = 0
	      end do
	   end do
	   i = 0
	   do while( .true. )
	      i = i + 1
	      if( i .gt. mline ) then
	         write( stdout, * ) 'LINLTE>> Too many lines in ',
     >                              'VCSSB table.'
	         write( stdout, * ) '         Not more than ', mline-1,
     >                              ' please.'
	         call exit
	      end if
	      read( vcssb_dat, *, end = 1 ) nl(i), nu(i),
     >           mne(i), mtemp(i), mp(i)
	      if( mp(i)     .gt. pmp .or.
     >            mne(i)    .gt. pmne .or.
     >            mtemp(i)  .gt. pmt ) then
	         write( stdout, * ) 'LINLTE>> VCSSB table too big in ',
     >                              'one of these:'
	         write( stdout, * ) '         mp: ', mp(i), ' >', pmp
	         write( stdout, * ) '         mne: ', mne(i), ' >', pmne
	         write( stdout, * ) '         mt: ', mtemp(i), ' >', pmt
 	         call exit
	      end if
	      if( nl(i) .gt. mij .or. nu(i) .gt. mij ) then
	         write( stdout, * ) 'LINLTE>> Too large a quantum ',
     >                              'number for ', i, 'th line in ',
     >                              'VCSSB table.'
	         call exit
	      end if
	      read( vcssb_dat, * ) (log_nel(j,i), j = 1, mne(i)),
     >                             log_t0(i), log_t_inc(i),
     >                             log_alpha0(i), log_alpha_inc(i)
	      ij( nl(i), nu(i) ) = i
	      read( vcssb_dat, * ) (( (svcs(m,j,k,i), k = 1, mp(i)),
     >                           m = 1, mtemp(i)),
     >                             j = 1, mne(i))
	   end do
1	   continue
	   close( vcssb_dat )
	   write( stdout, * ) 'VCSSB.DAT  OK!'
	end if ! first


	line = ij( lu(kma), int(diu(kma)) )	! ij(nlow, nup)	
	if( line .eq. 0 ) then
	   write( stdout, * ) 'LINLTE>>  No VCSSB data for line #',
     >                        kma
	   call exit
	end if

C ***
C ***	Temperature and electron density interpolation
C ***
	if( t .ne. old_t .or. ne .ne. old_ne .or.
     >      line .ne. old_line) then
           old_t = t
	   old_ne = ne
           old_line = line
           log_t = log10( t )
	   ne0 = exp10( log_nel(1,line) )

           btemp = ( log_t - log_t0(line) )/log_t_inc(line) + 1.
           itemp = max( min(int(btemp), mtemp(line)-1), 1 )
           wttemp = btemp - itemp

           log_ne = log10( ne )
	   do ine = 1, mne(line)
	      if( log_nel(ine,line) .gt. log_ne ) then
                 go to 2	! exit
	      end if
	   end do
2	   continue
           ine = ine - 1
           ine = max( min(ine, mne(line)-1), 1 )
           wtne = (log_ne - log_nel(ine,line))/
     >            (log_nel(ine+1,line) - log_nel(ine,line))
           if( wtne .lt. 0. ) wtne = 0.    ! no extrapolation
           if( wtne .gt. 1. ) wtne = 1.

           do i = 1, mp(line)
              prof( i ) = ( 1. - wtne )*( 1. - wttemp )*
     >                    svcs( itemp, ine, i, line ) +
*
     >                    ( 1. - wtne )*wttemp*
     >                    svcs( itemp + 1, ine, i, line ) + 
*
     >                    wtne*( 1. - wttemp )*
     >                    svcs( itemp, ine + 1, i, line ) +
*
     >                    wtne*wttemp*
     >                    svcs( itemp + 1, ine + 1, i, line )
	   end do
	end if

C ***	alpha/lambda interpolation

        f0 = 1.25e-9*max(ne,ne0)**0.666666666666666666e0  ! fix at table limit
	dl = abs( dlam )
        if( dl .ne. 0. ) then
           log_alpha = log10( dl/f0 )
           balp = ( log_alpha - log_alpha0(line) )/log_alpha_inc(line) +
     >            1.
           ialp = balp
           if( ialp .lt. 1 ) then
              vcssb = exp10( prof(1) )
           else if( ialp .gt. mp(line) - 1 ) then
              vcssb = exp10( prof(mp(line)) +
     >                     2.5*( log_alpha0(line) +
     >                           (mp(line)-1)*log_alpha_inc(line) -
     >                           log_alpha )
     >                   )
!	type *, 'VCSSB>  Extrapolated'
           else
              wt = balp - ialp
              vcssb = exp10( (1. - wt)*prof(ialp) +
     >                     wt*prof(ialp + 1) )
           end if
        else
           vcssb = exp10( prof(1) )
        end if

	vcssb = vcssb/f0
!	vcssb = vcssb*1.7724539	! normalize like Voigt function (=sqrt(pi))

        end
