        subroutine atmos
C ***
C ******************************************************************************
C ***
C        *****************************************************
C        *                                                   *
C        *   READIN OF MODEL-ATMOSPHERE FROM FILE            *
C        *   CALCULATING MASS-SCALE                          *
C        *   CALCULATING LTE-POPULATIONS OF ELEMENT          *
C        *   AND BACKGROUNG-OPACITIES                        *
C        *                                                   *
C        *       14-MAY-1991 18:02:50.79                     *
C        *                                                   *
C        *****************************************************
C ***
C ***   Modifications:
C ***
C ***   14-FEB-1994 18:21:33.00         ML / IoA.  Added H line opacity
C ***	13-MAY-1995 16:27:21.00		ML / IoA.  Completely restructured
C ***					and zeta's added for other elements
C ***	14-MAY-1995 19:28:57.00		ML / IoA.  Save ionization potentials
C ***	19-MAY-1995 19:59:40.00		ML / IoA.  Have zeta(...,0:1,...) and
C ***					ionpot(0:1,...)
C ***	25-APR-1996 15:30:11.62		ML / Bamberg.  Extend to 4 ion stages
C ***	29-APR-1996 15:17:47.40		ML / Bamberg.  Have chi_j(0:4) instead
C ***                                   of (1:6).  chi_j(1) wasn't used anywhere
C ***
C ******************************************************************************
C ***
        implicit none

	include 'MOD_SIZ.INC'

C ***
C ***	Global variables
C ***
	include 'MODEL.INC'
	include 'POP.INC'
	include 'NNS.INC'
	include 'LEVEL.INC'
	include 'LINE.INC'
	include 'OPAC.INC'

C ***	COMMON FOR SUBROUTINE IONDIS AND OPALAM
C ***
	include 'OPALION.INC'

	real aln10, boltzk, hplanc, clight
        common /const/ aln10, boltzk, hplanc, clight

C ***
C ***	Local variables
C ***
	integer f_atm
	parameter( f_atm = 21 )

	character c1, zeile*133

	integer i, id, iel, il, ion, it, j, jel0, jel, k, nxt

	real ab, abl, abui(m_mai), as, dnc, glog, hkev, hktev,
     >       natomt(m_depth), okap, osig, pele, pgas, reflam,
     >       reftau(m_depth), refkap(m_depth), refsig(m_depth),
     >       r(m_depth), rsu, sum, teff, x

C ***
C ***	Functions
C ***
	real xp

        xp( x ) = exp( x*2.30258 )

C ***
C ***	Code
C ***

C ***	CONST FOR SUBROUTINE OPALAM AND IONDIS
C ***
        aln10 = log( 10. )
        boltzk = 1.38062E-16
        hplanc = 6.6262E-27
        clight = 2.997925E+10

	c1 = char(1)

C***********************************************************************
C---READIN MODEL-ATMOSHERE FROM FILE------------------------------------
C***********************************************************************

        open( f_atm, file = 'ATM' )
        rewind f_atm

C ***	FIRST: ABUNDANCE DATA
        read( f_atm, 9000 ) mai
        as = 0.
        do i = 1, mai
           read( f_atm, 9000 ) nami( i ), wti( i ), abl
           ab = xp( abl )
           abui( i ) = ab
           fraci( i ) = ab
           as = as + ab
C ***
C ***	   SAVE ELEMENT NUMBER
C ***
           if( nami(i)/100 .eq. label(nk + 2)/100 ) iel = i
        end do

        he = abui( 2 )/abui( 1 )
        wei = wti( iel )

        do i = 1, mai
           fraci( i ) = fraci( i )/as
	end do

        avemas = 0.
        do i = 1, mai
           avemas = avemas + wti( i )*fraci( i )
	end do
        avemas = avemas*1.660531E-24
        avemat = avemas			! Save AVEMAS for IONDIS and OPALAM

C ***	OUTPUT ABUNDANCE DATA
	write( 6, '(/)' )
        write( zeile, 9010 ) c1, label( nk + 2 ),
     >                       c1, log10( abui(iel) ),
     >                       c1, fraci( iel ), c1, wei,
     >                       c1, avemas
	call sout( 6, zeile, c1 )

        read( f_atm, 9020 ) ndepth, teff, glog, reflam, rsu
        read( f_atm, 9030 ) ( reftau(id), id = 1, ndepth )
        read( f_atm, 9040 )
        read( f_atm, 9050 ) ( temp(id), id = 1, ndepth )

C ***	TEMPERATURE READIN AS THETA
        do id = 1, ndepth
           temp( id ) = 5040./temp( id )
        end do

        read( f_atm, 9040 )
        read( f_atm, 9030 ) ( pelg(id), id = 1, ndepth )
        read( f_atm, 9040 )
        read( f_atm, 9030 ) ( pglg(id), id = 1, ndepth )
        read( f_atm, 9040 )
        read( f_atm, 9030 ) ( refkap(id), id = 1, ndepth )
        read( f_atm, 9040 )
        read( f_atm, 9030 ) ( refsig(id), id = 1, ndepth )
        read( f_atm, 9040 )
        read( f_atm, 9030 ) ( r(id), id = 1, ndepth )
        read( f_atm, 9040 )
        read( f_atm, 9030 ) ( xi(id), id = 1, ndepth )
        rewind f_atm
        close( f_atm )

C***********************************************************************
C---CALCULATE MASS SCALE------------------------------------------------
C***********************************************************************

        do id = 1, ndepth
           refkap( id ) = avemas/( xp(refkap(id)) + xp(refsig(id)) )
        end do
        call integ( reftau, refkap, m, ndepth, refkap(1)*reftau(1) )

C***********************************************************************
C---CALCULATE POPULATIONS AND BACKGROUND-OPACITIES----------------------
C***********************************************************************

        do j = 1, 100
           jiz( j ) = 0	       ! OPALAM does not initialize this array, argh!
        end do

        hkev = 1.602192E-12/boltzk

        do 2 id = 1, ndepth

           pele = xp( pelg(id) )
           call iondis( temp(id), pele, 2, 1, pgas, dnc )

           natomt( id ) = dnc*fraci( iel )	! TOTAL POPULATION OF ELEMENT
           rho( id ) = avemas*dnc

           if( id .eq. 1 ) then
C ***
C ***	      SAVE ELEMENT NUMBERS AND IONIZATION-POTENTIAL
C ***
              jel0 = jiz( label(nk + 2)/100 )	     ! start of neutrals
              jel = jel0 + mod( label(nk + 2), 100 ) ! 1st state in model atom
              do ion = 0, 4
                 chi_j( ion ) = chij( jel0 + ion )
	      end do

	      k = 0
              do j = 1, 90
                 izp(j) = 0                          ! no such element
                 if( jiz(j) .gt. 0 ) then            ! this element does exist
                    k = k + 1                        ! count it
                    izp(j) = k                       ! save it
                    do ion = 0, 4                    ! for all ion stages
	               ionpot(ion,k) = chij( jiz(j)+ion )
                    end do
                 end if
	      end do
           end if

C ***
C ***	   HYDROGEN POP GROUND STATE----------
C ***	   and H II, i.e. protons
C ***	   And all the other elements, only first two ionization stages
C ***
C ***	   zeta(id,0,izp(Z))   is zeta for element Z (e.g., 26), neutrals
C ***	   zeta(id,1,izp(Z))   is zeta for element Z (e.g., 26), singly ioniz.
C ***
           do j = 1, 90
              if( jiz(j) .gt. 0 ) then            ! this element does exist
                 do ion = 0, 4                    ! save it for all ion stages
                    zeta( id, ion, izp(j) ) = zetaj( jiz(j)+ion )
                 end do
              end if
	   end do


C ***	   Level pop for element.
C ***      Implemented all 5 ionization stages opalam provides.
C ***	   Assumption here is that each ground state ionizes into the
C ***      ground state of the next ionization stage.
C ***
C ***	   You can only go to x IV though as you need the continuum of
C ***      the next stage.  There is currently no x VI in opalam.
C ***
           hktev = hkev/temp( id )
           sum = 0.
           il = 1      ! first level of model atom
           j = jel     ! first ion stage in model atom (as pointer into zetaj)

           do while( il .lt. nk )
              do i = il, nxtion(il) - 1 ! loop over all levels of this ion stage
                 n( id, i ) = g( i )*zetaj( j )*dnc*
     >                        exp( -hktev*eps(i) )
                 sum = sum + n( id, i )
              end do
              il = nxtion( il )         ! next ion stage (level #)
              j = j + 1                 ! ditto          (pointer into zetaj)
           end do

           n( id, nk ) = fracj( j )*dnc	 	 ! continuum population
           natom( id ) = sum + n( id, nk )	 ! used total pop of element

C ***
C ***      background opacities for each line of model atom
C ***
           do it = 1, nt
              call opalam( lamt(it,0), temp(id), pele, dnc, 1, okap,
     >                     osig )
              akl( id, it ) = okap
              asl( id, it ) = osig
              call hlinop( lamt(it,0), temp(id), pele, okap )
              akl( id, it ) = akl( id, it ) + okap
           end do
2       continue

C***********************************************************************
C---MODEL ATMOSPHERE OUTPUT---------------------------------------------
C***********************************************************************

        do id = 1, ndepth
           if( mod(id, 50) .eq. 1 ) then
	      write( 6, '(a,/)' ) '1Model atmosphere read in from file'
	      write( zeile, 9060 ) c1, teff, c1, glog, c1, he,
     >                             c1, rsu, c1, r(1)*1e-5
	      call sout( 6, zeile, c1 )
	      write( 6, 9065 ) reflam
           end if
           write( 6, 9070 ) id, log10( reftau(id) ), m(id),
     >                      log10( m(id) ), temp(id), pelg(id),
     >                      pglg(id), rho(id), xi(id), r(id)/r(1),
     >                      natomt(id), natom(id)
        end do
        if( rsu .le. 0. ) write( 6, '(//,a)' )
     >                    ' Assuming constant log g'
        write( 6, '(''1'')' )

9000    format( i7, f9.4, f7.3 )
9010    format( ' Element: ', a, i6, '  log(Abund/H) = ', a, f8.3,
     >        '  Abund/nKern = ', a, e12.5, '  Weight = ', a, f8.3,
     >        '  AVEMAS = ', a, e15.5 )
9020    format( ///1x, i4, /1x, f8.1, f6.2, f10.2, f7.2, // )
9030    format( 1x, 5e14.6 )
9040    format( // )
9050    format( 1x, 5f14.6 )

9060    format( ' Temp = ', a, f7.0, 4x, 'log g = ', a, f7.2, 4x, 
     >          'He/H = ', a, f5.3, 4x, 'r/r(Sun) = ', a, f7.2, 3x, 
     >          '(', a, 1p, e11.3, ' km)' )
9065    format( //5x,
     >        'tau(', 0p, f6.0, ')', 6x, 'Mass', 5x, 'lg Mass', 3x,
     >        'Temp', 4x, 'lg pe', 3x, 'lg pg', 7x, 'rho', 6x, 'micro',
     >        4x, 'r/r*', 2x, 'Tot-FRACI*rho', 1x, 'Mod-FRACI*rho' )

9070    format( i4, 0p, f11.3, 1p, e15.5, 0p, f8.3, f8.0, 2f8.3, 1p,
     >        e14.5, 0p, f7.2, f9.4, 1p, 2e14.5 )

        end
