	SUBROUTINE HECLRT( IID, C, ICOL )
C ***
C ******************************************************************************
C ***
C ***	Computes collison frequencies for helium using formalism from
C ***	K. Werner's program.
C ***
C ***	14-MAY-1991 18:05:52.22
C ***	13-MAY-1995 17:40:41.00		ML / IoA.  Include files, zetah
C ***	29-APR-1996 16:47:12.90		ML / Bamberg.  New ion stuff.
C ***	17-MAY-1996 13:53:32.56		ML / Bamberg.  ICOL to parameter list
C ***
C ******************************************************************************
C ***

C ***
C ***	Arguments
C ***
	integer iid, icol
	real c(icol,icol)

        INCLUDE 'MOD_SIZ.INC'

	PARAMETER( EVLAM = 12398.54, E1H = 13.598, PIA02 = 8.79737E-17 )
C ***
	INTEGER SCRATCH
	PARAMETER( SCRATCH = 17 )
C ***
	PARAMETER( MAXP = 8 )
	INTEGER FORMULA
	REAL NE, NHI0, NEFF, LOG10T
	real P(MAXP)
	LOGICAL FIRST
	DATA FIRST /.TRUE./
	SAVE FIRST
C ***
	INCLUDE 'COLIS.INC'
	CHARACTER*80 HE_COLL, HE_COLNEU
	COMMON /HECOLC/ HE_COLL(M_COLL), HE_COLNEU(M_COLL)
	COMMON /HECOL/ H1BB, H1BF, ELBB, ELBF
	include 'LINE.INC'
	include 'LEVEL.INC'
	include 'NNS.INC'
	include 'MODEL.INC'
	INCLUDE 'POP.INC'

C ***
C ***	Definition of global values
C ***
	ID=IID
	T= TEMP(ID)
	TEV=8.6171E-5*T
	NE=10.**(PELG(ID))/T/1.38062E-16
	NHI0=2.*RHO(ID)/AVEMAS*ZETAH(ID)
	ACON=NE*5.8875E-9*SQRT(TEV)	   ! ne*pi*a0**2*sqrt(8/pi*m)*sqrt(k*T)
	BCON=NHI0*5.93095E-13*SQRT(TEV*WEI*(WEI+1.008))
C ***
C ***	Initialize collisions to zero
C ***
	DO 11 I = 1, NK
	   DO 12 J = 1, NK
	      C(I,J) = 0.
12	   CONTINUE
11	CONTINUE
C ***
C ***	Electron collisions
C ***
	I = 0
1	CONTINUE
	   I = I + 1
	   IF( HE_COLL(I) .EQ. 'END' ) GO TO 2
	      READ( HE_COLL(I), '(4i3, 3e12.5, 5f6.3 )' ) ILOW, IUP,
     >	  	FORMULA, IP, (P(J), J = 1, IP )
	      IF( IUP .LT. NXTION(ILOW) ) THEN
C ***
C ***		 Bound - bound collision
C ***
		 E = EPS(IUP) - EPS(ILOW)
		 E1HE = (E1H/E)*(E1H/E)
		 U0 = E/TEV
C ***
		 IF( FORMULA .EQ. 2 ) THEN
C ***
C ***		    He I forbidden transitions, Omega = const. = 1
C ***
		    IF( IP .NE. 0 ) CALL HEERR( ILOW, IUP, IP, 0 )
		    C(ILOW,IUP) = 8.631E-6/G(ILOW)/SQRT(T)*NE*EXP(-U0)
		 ELSE IF( FORMULA .EQ. 5 ) THEN
C ***
C ***		    He I allowed transitions from ground state (Mihalas,
C ***								Stone)
C ***
		    IF( IP .NE. 1 ) CALL HEERR( ILOW, IUP, IP, 1 )
		    FIJ = P(1)
		    C(ILOW,IUP) = ELBB*ACON*4.*FIJ*E1HE*U0*EXPINT(U0)
		 ELSE IF( FORMULA .EQ. 6 ) THEN
C ***
C ***		    He I allowed transitions from exited states (Mihalas,
C ***								Stone)
C ***
		    IF( IP .NE. 1 ) CALL HEERR( ILOW, IUP, IP, 1 )
		    FIJ = P(1)
		    U1 = U0 + 0.2                                 
		    C(ILOW,IUP) = ELBB*ACON*4.*FIJ*E1HE*U0*(
     >		      EXPINT(U0) - U0/U1*0.819*EXPINT(U1))
		 ELSE IF( FORMULA .EQ. 7 ) THEN
C ***
C ***		    He I forbidden transitions from ground state (Mihalas,
C ***								  Stone)
C ***
		    IF( IP .NE. 8 ) CALL HEERR( ILOW, IUP, IP, 8 )
		    A = P(1)
		    A0 = P(2)
		    A1 = P(3)
		    B1 = P(4)
		    C1 = 1./P(5)
		    A2 = P(6)
		    B2 = P(7)
		    C2 = 1./P(8)
		    NEFF = 2.*SQRT(E1H/E)
		    NEFF = NEFF*NEFF*NEFF
		    X1 = (B1*(U0 + C1) + 2.)/(U0 + C1)**3
		    X2 = (B2*(U0 + C2) + 2.)/(U0 + C2)**3
		    C(ILOW,IUP) = ELBB*ACON*A/NEFF/PIA02*U0*
     >		      ( A0*EXPINT(U0) + A1*U0*X1*EXP(-B1*U0) +
     >		      A2*U0*X2*EXP(-B2*U0) )
		 ELSE IF( FORMULA .EQ. 8 ) THEN
C ***
C ***		    He I forbidden transitions between n = 2 sublevels (Mihalas,
C ***									Stone)
C ***
		    IF( IP .NE. 3 ) CALL HEERR( ILOW, IUP, IP, 3 )
		    C0 = P(1)
		    C1 = P(2)
		    C2 = P(3)
		    LOG10T = LOG10(T)
		    GAMMA = 10.**(C0 + C1*LOG10T + C2/(LOG10T*LOG10T))
		    C(ILOW,IUP) = ELBB*ACON*EXP(-U0)*GAMMA
		 ELSE IF( FORMULA .EQ. 9 ) THEN
C ***
C ***		    Unknown cross seqtions. pi a0**2 used.
C ***
		    IF( IP .NE. 0 ) CALL HEERR( ILOW, IUP, IP, 0 )
		    C(ILOW,IUP) = ELBB*ACON*EXP(-U0)*(1. + U0)
		 ELSE
		    WRITE( 6, * ) 'HECLRT>> Formula No. ', FORMULA,
     >		      ' not implemented for bound - bound electron',
     >		      ' collisions!'
		    STOP
		 END IF
	      ELSE
C ***
C ***		 Bound - free collision
C ***
		 E = (CHI_J(0) - EPS(ILOW) + EPS(IUP) + DI(ILOW))
		 U0 = E/TEV
C ***
		 IF( FORMULA .EQ. 3 ) THEN
C ***
C ***		    He I from ground state up to n = 15
C ***
		    IF( IP .NE. 1 ) CALL HEERR( ILOW, IUP, IP, 1 )
		    U1 = U0 + 0.27
		    U2 = U0 + 1.43
		    S0 = P(1)
		    C(ILOW, IUP) = ELBF*ACON*S0*(U0*EXPINT(U0) -
     >		      0.728*U0*U0/U1*EXPINT(U1) - 0.189*U0*U0*EXP(-U0)*
     >		      ((2. + U2)/(U2*U2*U2)))
		 ELSE IF( FORMULA .EQ. 6 ) THEN
C ***
C ***		    Other states with usual formula of Drawin
C ***
		    IF( IP .NE. 0 ) CALL HEERR( ILOW, IUP, IP, 1 )
		    C(ILOW, IUP) = ELBF*ACON*4.*U0*EXP(-U0)/(1. + U0)*
     >	  	      (1./(20.+U0) + LOG( 1.25*(1. + 1./U0) ))*
     >	  	      0.665*(E1H/E)*(E1H/E)*1.
		 ELSE
		    WRITE( 6, * ) 'HECLRT>> Formula no. ', FORMULA,
     >		      ' not implemented for bound - free electron',
     >		      ' collisions!'
		    STOP
		 END IF
	      END IF
C ***
C ***	      Downward rate
C ***
	      C(IUP,ILOW) = C(ILOW,IUP)*N(ID,ILOW)/N(ID,IUP)
	   GO TO 1
2	CONTINUE
C ***
C ***	Neutral particle collisions
C ***
	IF( FIRST ) THEN
	   WRITE( 6, * ) 'HECLRT>> No neutral particle collisions ',
     >	     'implemented!'
	   FIRST = .FALSE.
	END IF
	RETURN
	END
