        function isamax( n, sx, incx )
C ***
C ******************************************************************************
C ***
C ***   Substitute for Cray function.
C ***
C ***
C ***     NAME
C ***          ISAMAX, ICAMAX - Finds first index of largest absolute value
C ***          in vectors
C ***
C ***     SYNOPSIS
C ***          imax = ISAMAX(n,sx,incx)
C ***
C ***          imax = ICAMAX(n,cx,incx)
C ***
C ***     DESCRIPTION
C ***          n         Number of elements to process in the vector to be
C ***                    searched
C ***                    (n = vector length if incx = 1; n = vector
C ***                    length/2 if incx = 2, and so on).
C ***                    If n <= 0, ISAMAX and ICAMAX return 0.
C ***
C ***          sx        Real vector to be searched
C ***
C ***          cx        Complex vector to be searched
C ***
C ***          incx      Increment between elements of sx or cx; for
C ***                    contiguous elements, incx = 1.
C ***
C ***          These integer functions find the first index of the largest
C ***          absolute value of the elements of a vector.
C ***
C ***          ISAMAX returns the first index i such that
C ***
C ***          | ( x sub i ) | = max | ( x sub j ) | : j = 1, . . . ,n
C ***
C ***          where ( x sub j ) is an element of a real vector.
C ***
C ***          ICAMAX determines the first index i such that
C ***
C ***          | Real( x sub i ) | + | Imag( x sub i ) | =
C ***
C ***          max | Real( x sub j ) | + | Imag( x sub j ) | :
C ***
C ***          j = 1, . . . ,n
C ***
C ***          where ( x sub j ) is an element of a complex vector.
C ***
C ***
C ***
C ***   19-DEC-1990 20:06:30.78
C ***
C ******************************************************************************
C ***
        real sx(n*incx)

        if( n .gt. 0 ) then
           j = 1
           rmax = 0.
           do i = 1, n
              rmax = max( rmax, abs(sx(j)) )
              j = j  + incx
           end do
           j = 1
           do i = 1, n
              if( abs(sx(j)) .eq. rmax ) then
                 isamax = j
                 return
              end if
              j = j  + incx
           end do
        else
           isamax = 0
        end if
        end
