	function qab( t, sig, par, n )
*	function qab( t, sig, gwe, te, we, wexe, ae, be, n )
C ***
C ******************************************************************************
C ***
C ***	Compute internal molecular partition function
C ***	according to Tatum (1966), Publ. Domin. Astr. Obs. 13, No 1,
C ***	formula 15
C ***
C ***	t	r >	Temperature to compute Qab for
C ***	sig	r >	Symmetry factor (1 for heteronulcear molecules, 
C ***				         2 for homonuclear molecules)
C ***   par     r >     the other six parameters required for each level i:
C ***			par(1,i)   gwe	Electronic statistical weight
C ***			par(2,i)   Te	term value of electronic state [cm^-1]
C ***			par(3,i)   we	fundamental frequency omega_e [cm^-1]
C ***			par(4,i)   wexe	[cm^-1]
C ***			par(5,i)   ae	rotation-vibration interaction constant alpha_e [cm^-1]
C ***			par(6,i)   be	rotation constant [cm^-1]
C ***			This should rather be a struct but Fortran 77 doesn't
C ***                   have them.
C ***	n	i >	number of states
C ***
C ***
C ***	24-FEB-1998 17:37:29.36		ML / Bamberg.  Made.
C ***
C ******************************************************************************
C ***
	implicit none

C ***
C ***	Arguments
C ***
	integer n
!	real t, sig, gwe(n), te(n), we(n), wexe(n), ae(n), be(n)
        real t, sig, par(6,n)
	real qab

C ***
C ***	Local variables
C ***
	real hck, te_max
	parameter( hck = 6.6261e-34*2.99797e10/1.3807e-23,	! h*c/k
     >             te_max = 40 000.
     >           )

	real hckt		! hc/kT
	integer i, iv, ivmax, imin


C ***
C ***	Code
C ***

	qab = 0.
	imin = 1
	hckt = hck/T

	do i = 1, n
C ***
C ***	   Sum over states with energy <= te_max
C ***
	   if( par(2,i) .le. te_max ) then
	      ivmax = int( par(3,i)/2./par(4,i) - 0.5 )
              if( par(2,i) .lt. par(2,imin) ) imin = i	! find ground state
	      do iv = 0, ivmax
C ***
C ***	         Sum over all vibrational states
C ***
                 qab = qab + exp( -(par(3,i)*(iv+0.5) - 
     >                              par(4,i)*(iv+0.5)*(iv+0.5) + 
     >                              par(2,i)
     >                             )*hckt
     >                          )/(par(6,i) - par(5,i)*(iv+0.5))*
     >                       par(1,i)/sig/hckt
	      end do
	   end if
	end do

	qab = qab*exp( hckt*(0.5*par(3,imin) - 0.25*par(4,imin)) )


	end
