      SUBROUTINE QMOL(TEMP)
C     PARTITION FUNCTIONS:MOLECULES
C     DISSOCIATION CONSTANTS SAHAJ:P(A)*P(B)/P(AB)
C ***
C *** Modifications:
C ***
C ***   11-DEC-1991           		ML / UT Austin.  OH added
C ***    8-JAN-1992 	    		ML / UT Austin.  TiO added
C ***	17-FEB-1998 19:14:04.98		ML / Bamberg.  Resurrected from 
C ***					crown.as.utexas.edu
C ***					Was a .bak file with a typo in
C ***					TiO constant.
C ***
C ***					Add NH, SiH, SiO
C ***	25-FEB-1998 18:05:33.23		ML / Bamberg.  Replace all Tsuji
C ***					formulae (OH, TiO, NH, SiH, SiO)
C ***					with explicit calculations from
C ***					molecular constants.  See function QAB
C ***	26-FEB-1998 18:05:19.76		ML / Bamberg.  Replace TiO partition
C ***					function with Jrgensen 1998 data.
C ***
      implicit none

      real temp
 
      integer i, j, j1, j2, ier
      real th, logth, tn, t25, d0, rm
      real x

      real tj(15), qcnj(15), qchj(15), tjtio(40), qtioj(40)  ! Jrgensen data

      real aln10, boltzk, hplanc, clight
      COMMON/CONST/ALN10,BOLTZK,HPLANC,CLIGHT

      include 'OPALION.INC'

C ***
C *** Functions
C ***
      external intep

      data tj /1000., 1500., 2000., 2500., 3000., 3500., 4000., 4500.,
     >         5000., 5500., 6000., 6500., 7000., 7500., 8000. / ! T Jrgensen
      data tjtio /200., 400., 600., 800., 1000., 1200., 1400., 1600.,
     >            1800., 2000., 2200., 2400., 2600., 2800., 3000.,
     >            3200., 3400., 3600., 3800., 4000., 4200., 4400.,
     >            4600., 4800., 5000., 5200., 5400., 5600., 5800.,
     >            6000., 6200., 6400., 6600., 6800., 7000., 7200.,
     >            7400., 7600., 7800., 8000./ 	! T Jrgensen TiO

C ***
C *** Code
C ***
      TH=5040./TEMP
      TN=-TH*ALN10
      logth = log10(th)
      x = min( log10(5040./1000.), max( logth, log10(5040./9000.) ) )
      T25=4.6968E13/TH/TH/SQRT(TH)
      J=JNEGI
C
C     H2 Irwin 1987, Table 8 + Formula 48
      J=J+1
      NAMJ(J)=01000100
      WTJ(J)=2.016
      CHIJ(J)=15.4258 ! Huber & Herzberg
      RM=0.50391261   ! Huber & Herzberg
      D0=4.4781       ! Huber & Herzberg
      PARTFJ(J) = exp( (1.69179e0 + (-1.72270e0 + 
     >            (7.98033e-1 + (-1.57089e-1 + (-5.35313e-1 +
     >            (1.75818e0 + (-2.63895e0 + 1.357080e0*x)*
     >            x)*x)*x)*x)*x)*x)*aln10 )
      J1=JIZ(1)
      J2=JIZ(1)
      SAHAJ(J)=RM*SQRT(RM)*T25*PARTFJ(J1)*PARTFJ(J2)/partfj(j)*
     >         EXP(D0*TN)
C
C     CH Jrgensen 1996 AA 315, 204, Tab 1, col 6
      J=J+1
      NAMJ(J)=06000100
      WTJ(J)=13.019
      CHIJ(J)=10.64  ! Huber & Herzberg
      RM=0.92974056  ! Huber & Herzberg
      D0=3.46        ! Huber & Herzberg
      data qchj /203.5, 324.7, 471.2, 646.2, 850.8, 1085.1, 1348.1,
     >           1638.7, 1955.3, 2296.1, 2659.3, 3043.2, 3446.2, 3866.8,
     >           4303.7/
      call intep( temp, partfj(j), tj, qchj, 15, ier )
      J1=JIZ(6)
      J2=JIZ(1)
      SAHAJ(J)=RM*SQRT(RM)*T25*PARTFJ(J1)*PARTFJ(J2)/partfj(j)*
     >         EXP(D0*TN)
C
C     C2 Sauval & Tatum 1984
      J=J+1
      NAMJ(J)=06000600
      WTJ(J)=24.022
      CHIJ(J)=12.15  ! Huber & Herzberg
      RM=6.003
      D0=6.21	     ! Huber & Herzberg
      PARTFJ(J) = exp( (4.3091e0 + (-2.24060e0 + 
     >            (0.4865 - 0.2049*x)*x)*x)*aln10 )
      J1=JIZ(6)                         
      J2=JIZ(6)
      SAHAJ(J)=RM*SQRT(RM)*T25*PARTFJ(J1)*PARTFJ(J2)/partfj(j)*
     >         EXP(D0*TN)
C
C     CN Jrgensen & Larsson 1990, AA 238, 424; Tab 1, col 7
      J=J+1
      NAMJ(J)=06000700
      WTJ(J)=26.018
      CHIJ(J)=14.1   ! Huber & Herzberg
      RM=6.46219329  ! Huber & Herzberg
      D0=7.77        ! Costes, Naulin, & Dorthe, 1990, AA 232,270
      data qcnj /779.3, 1292.3, 1932.0, 2714.9, 3664.2, 4809.5, 6183.5,
     >           7819.6, 9750.5, 12008.1, 14622.9, 17624.1, 21040.5,
     >           24899.6, 29228.4/
      call eintep( temp, partfj(j), tj, qcnj, 15, ier ) ! same TJ as above
                                                        ! so we can use EINTEP
      J1=JIZ(6)
      J2=JIZ(7)
      SAHAJ(J)=RM*SQRT(RM)*T25*PARTFJ(J1)*PARTFJ(J2)/partfj(j)*
     >         EXP(D0*TN)
C
C     CO Irwin 1987, Table 8 + Formula 48
      J=J+1
      NAMJ(J)=06000800
      WTJ(J)=28.010
      CHIJ(J)=14.0139 ! Huber & Herzberg
      RM=6.85620871   ! Huber & Herzberg
      D0=11.09        ! Huber & Herzberg
      PARTFJ(J) = exp( (3.61530e0 + (-1.773848e0 + 
     >            (3.516181e-1 + (8.620792e-2 + (2.911791e-1 +
     >            (-1.141469e0 + (2.513133e0 + (-2.886502e0 +
     >             1.238932e0*x)*x)*
     >            x)*x)*x)*x)*x)*x)*aln10 )
      J1=JIZ(6)
      J2=JIZ(8)
      SAHAJ(J)=RM*SQRT(RM)*T25*PARTFJ(J1)*PARTFJ(J2)/partfj(j)*
     >         EXP(D0*TN)
C
C     MgH  Sauval & Tatum 1984
      J=J+1
      NAMJ(J)=12000100
      WTJ(J)=25.313
      CHIJ(J)=7.6
      RM=0.96718516 ! Huber & Herzberg
      D0=1.34       ! Huber & Herzberg
      PARTFJ(J) = exp( (3.6704e0 + (-2.26820e0 + 
     >            (0.9354 - 0.2597*x)*x)*x)*aln10 )
      J1=JIZ(12)
      J2=JIZ(1)
      SAHAJ(J)=RM*SQRT(RM)*T25*PARTFJ(J1)*PARTFJ(J2)/partfj(j)*
     >         EXP(D0*TN)
C
C     OH
C     DISSOCIATION CONSTANTS: Sauval & Tatum 1984
      J = J + 1
      NAMJ(J) = 08000100
      WTJ(J) = 17.00734
      CHIJ(J) = 12.9   ! Huber & Herzberg
      rm = 0.94808719  ! Huber & Herzberg
      d0 = 4.392       ! Huber & Herzberg
      PARTFJ(J) = exp( (3.0929e0 + (-1.67780e0 + 
     >            (0.6743 - 0.1874*x)*x)*x)*aln10 )
      J1=JIZ(8)
      J2=JIZ(1)
      SAHAJ(J)=RM*SQRT(RM)*T25*PARTFJ(J1)*PARTFJ(J2)/partfj(j)*
     >         EXP(D0*TN)
C
C     TiO
C     DISSOCIATION CONSTANTS: Jrgensen 1994, AA 284, 179
      J = J + 1
      NAMJ(J) = 22000800
      WTJ(J) = 63.8794
      CHIJ(J) = 6.4    ! Huber & Herzberg
      rm = 11.9938851  ! Huber & Herzberg
      d0 = 6.87        ! Huber & Herzberg
      data qtioj/
     > 8.645E+02, 2.297E+03, 4.090E+03, 6.290E+03, 8.932E+03, 1.204E+04,
     > 1.565E+04, 1.977E+04, 2.442E+04, 2.964E+04, 3.541E+04, 4.177E+04,
     > 4.871E+04, 5.622E+04, 6.432E+04, 7.299E+04, 8.222E+04, 9.200E+04,
     > 1.023E+05, 1.132E+05, 1.245E+05, 1.363E+05, 1.486E+05, 1.613E+05,
     > 1.745E+05, 1.880E+05, 2.020E+05, 2.163E+05, 2.309E+05, 2.458E+05,
     > 2.611E+05, 2.766E+05, 2.923E+05, 3.083E+05, 3.245E+05, 3.409E+05,
     > 3.575E+05, 3.742E+05, 3.911E+05, 4.081E+05/
      call intep( temp, partfj(j), tjtio, qtioj, 40, ier ) ! need INTEP here
      J1=JIZ(22)
      J2=JIZ(6)
      SAHAJ(J)=RM*SQRT(RM)*T25*PARTFJ(J1)*PARTFJ(J2)/partfj(j)*
     >         EXP(D0*TN)

C
C     NH
C     DISSOCIATION CONSTANTS: Sauval & Tatum 1984
      J = J + 1
      NAMJ(J) = 07000100
      WTJ(J) = 15.0147
      CHIJ(J) = 13.63  ! Huber & Herzberg
      rm = 0.94016928  ! Huber & Herzberg
      d0 = 3.47        ! Huber & Herzberg
      PARTFJ(J) = exp( (3.0735e0 + (-1.85010e0 + 
     >            (0.9607 - 0.3935*x)*x)*x)*aln10 )
      J1=JIZ(7)
      J2=JIZ(1)
      SAHAJ(J)=RM*SQRT(RM)*T25*PARTFJ(J1)*PARTFJ(J2)/partfj(j)*
     >         EXP(D0*TN)


C
C     SiH
C     DISSOCIATION CONSTANTS: Sauval & Tatum 1984
      J = J + 1
      NAMJ(J) = 14000100
      WTJ(J) = 28.0860 + 1.0080
      CHIJ(J) = 8.04    ! Huber & Herzberg
      rm = 0.97278226   ! Huber & Herzberg
      d0 = 3.06         ! Huber & Herzberg
      PARTFJ(J) = exp( (3.6908e0 + (-1.98010e0 + 
     >            (0.7704 - 0.2247*x)*x)*x)*aln10 )
      J1=JIZ(14)
      J2=JIZ(1)
      SAHAJ(J)=RM*SQRT(RM)*T25*PARTFJ(J1)*PARTFJ(J2)/partfj(j)*
     >         EXP(D0*TN)
C
C     SiO
C     DISSOCIATION CONSTANTS: Sauval & Tatum 1984
      J = J + 1
      NAMJ(J) = 14000800
      WTJ(J) = 28.0860 + 15.9994
      CHIJ(J) = 11.4	! Huber & Herzberg
      rm = 10.1767076   ! Huber & Herzberg
      d0 = 8.26         ! Huber & Herzberg
      PARTFJ(J) = exp( (4.2275e0 + (-1.91440e0 + 
     >            (0.7201 + (-1.3099 + 1.1657*x)*x)*x)*x)*aln10 )
      J1=JIZ(14)
      J2=JIZ(8)
      SAHAJ(J)=RM*SQRT(RM)*T25*PARTFJ(J1)*PARTFJ(J2)/partfj(j)*
     >         EXP(D0*TN)
C
      JMOL2=J
      RETURN
      END
