     	subroutine plotout( imode, xdata, ydata, ndata, style, color )
C ***
C ******************************************************************************
C ***
C ***	Routine to plot spectrum/fit.  Modeled after putout.
C ***
C ***	Has entry plotoutend to be called to finish.
C ***
C ***
C ***	IMODE	> I	which plot to make -- see comments in code
C ***
C ***
C ***	Plot data is passed as
C ***
C ***	xdata	r >	x values
C ***	ydata	r >	y values
C ***	ndata	i >	number of points in xdata/ydata
C ***	style	r >	line style (see diagr/over)
C ***	color	i >	color index
C ***
C ***	and frame limits via common /rect_size/ as
C ***
C ***	xmin	r >	mininum x value	(user coord)
C ***	xmax	r >	maximum x value	(user coord)
C ***	ymin	r >	minimum y value	(user coord)
C ***	ymax	r >	maximum y value	(user coord)
C ***
C ***
C ***	Frame labeling via common /my_cdata/ as
C ***
C ***	xlab	c >	label of x-axis
C ***	ylab	c >	label of y-axis
C ***	tlab	c >	label on top of frame
C ***
C ***
C ***	Modifications:
C ***
C ***	 4-JUN-1996 12:52:09.31		ML / Bamberg.  Pried from linlte.
C ***	19-JUN-1996 18:18:45.33		ML / Bamberg.  Changed calls to over to
C ***					ovr to allow for color.
C ***	 7-OCT-1996 18:59:26.33		ML / Bamberg.  Added call to psize to
C ***                                   allow for variable frame size via
C ***                                   environment variables.
C ***	20-FEB-1997 19:12:11.51		ML / Bamberg.  Pass xsize, ysize to
C ***                                   new_plot to make variable frame size
C ***                                   actually work.
C ***	 3-OCT-2001 14:28:23.00		ML / Stegaurach.  Changed arg list to
C ***					include plot data which was passed via
C ***					common.  Added entry plotoutend.
C ***
C ***
C ******************************************************************************
C ***
	implicit none

C ***
C ***	Arguments
C ***
	integer imode, ndata, color
	real xdata(ndata), ydata(ndata), style

C ***
C ***	Global variables
C ***
	include 'RESIZE.INC'
	include 'RECT_SIZE.INC'
	include 'ERASURE.INC'

C ***
C ***	Local variables
C ***

	logical plotted, hard, refresh, do_rect
	integer i, j, k, mode, status
	real xinc, yinc, x(4), y(4), x0, y0, size, xsize, ysize


C ***
C ***	Functions
C ***
	integer new_plot

	save plotted, hard, refresh, xsize, ysize

	data plotted /.false./



	mode = imode
	go to 1

	entry plotoutend()
C ***         ==========
	mode = 4


1	continue


	if( mode .eq. 1 ) then
C ***
C ***	   Make new plot
C ***
C ***	   pass plot size via xmin, xmax, ymin, ymax in common /rect_size/
C ***
C ***	   Stays in grafics mode on exit (or whatever your grafix package
C ***	   does after something has been plotted)
C ***
	   if( plotted ) then
              status = new_plot( xsize + 6., ysize + 5. )
	      if( status .eq. failed ) then
	         call pend
                 if( hard ) call phard
                 plotted = .false.
	      else if( status .eq. no_refresh ) then
	         refresh = .false.
                 return

	      end if
	   end if
	   if( .not.plotted ) then
              call psize( xsize, ysize )
	      call pbegin( xsize + 6., ysize + 5. )
              hard = itv .eq. 3
	      plotted = .true.
              refresh = .true.
	   end if

           xinc  = 0.
           yinc  = 0.

	   call plotscale( xdata, ydata, ndata )
     	   call diagr( 4.5, 3., xsize, ysize, xmin, xmax, ymin, ymax,
     >                 xinc, yinc, xlab, ylab, tlab, 1, 1, xdata,
     >                 ydata, 0, 0. )	! doesn't actually plot any data

	else if( .not.plotted ) then
C ***
C ***	   Programming error.  Must have opened plot first for 
C ***	   rest of the functions.
C ***
	   write( stdout, * ) 'Illegal call of PLOTOUT - must ',
     >                        'call PLOTOUT(1) first.'
           call exit
	else if( mode .eq. 3 ) then
C ***
C ***	   overplot new data
C ***
C ***	   switches text/grafix/text
C ***	   on entry; text 1 plane
C ***	   on exit:  text 1 plane
C ***
	   call gplane( 3, noErase, Single_plane )
	   call ovr( xdata, ydata, ndata, style, color )
	   call tplane( noErase, Single_plane )
	else if( mode .eq. 4 ) then
C ***
C ***	   finish plot
C ***
C ***	   on entry: grafix
C ***	   on exit:  text (or whatever your plot package does on finish)
C ***
	   call pend
           plotted = .false.
*	else if( mode .eq. 7 ) then
C ***
C ***	   write current fit parameters into plot (like putout(7))
C ***
!	   call ptext( xt, yt, text, sizet, loct, icol )
!		Darn, this can't be done independent of plot package.
!		Need a text plotting function (e.g. symblk).
!
	else

	   write( stdout, * ) 'Illegal argument to PLOTOUT', mode
	   call exit

	end if

	end
