	function new_plot( xsize, ysize )
C ***
C ******************************************************************************
C ***
C ***	Starts a new plot.
C ***	This is a graphics system dependent function.
C ***
C ***	If your system has the abilty to start a new plot on the screen
C ***	without closing the current plot and opening a new one, do so
C ***	here and return OK when sucessful.  On error or if your
C ***	graphics package is too stupid return FAILED and the functionality
C ***	will be implemented as close plot/open plot elsewhere.
C ***
C ***	You must also provide code to handle switching from screen to paper
C ***	and back.  In the most simple case you could simply return FAILED
C ***	and a close plot/print plot/open plot sequence will be issued elsewhere.
C ***
C ***
C ***	xsize, ysize	> R	size of plot in centimeters
C ***	new_plot	< I    	 failed	: couldn't do anything clever, need
C ***					  open/close plot sequence
C ***				     ok : could start new plot; implies screen
C ***					  needs to be repainted after paper
C ***					  plot was made
C ***			     no_refresh : could start new plot, don't repaint
C ***					  screen after paper plot -- this must
C ***					  never be returned for paper plots
C ***
C ***	Modifications:
C ***
C ***	 8-OCT-1994 14:10:14.00		ML / IoA.  Created.  Mongo version.
C ***	18-NOV-1994 17:50:19.00		ML / IoA.  Added screen/paper switch
C ***	21-NOV-1994 14:15:33.00		ML / IoA.  Made same more clever,
C ***					made function type integer
C ***	20-FEB-1997 19:09:07.81		ML / Bamberg.  Added xsize, ysize
C ***                                   arguments.
C ***
C ******************************************************************************
C ***
	implicit none

C ***
C ***	Arguments
C ***
	integer new_plot
	real xsize, ysize

C ***
C ***	Global variables
C ***
	include 'RESIZE.INC'
	include 'ERASURE.INC'

C ***
C ***	Local variables
C ***
	integer old_itv

	data old_itv /2/	! start out on screen
	save old_itv

C ***
C ***	Code
C ***

	if( itv .eq. 3 ) then

	   if( old_itv .ne. 2 ) then
C ***
C ***	      no switch, new paper plot
C ***
	      new_plot = failed	! need close/open thingy
	   else
C ***
C ***	      switch screen -> make new paper
C ***
	      call ploto( xsize, ysize, .false., .true. )    ! open paper plot
	      new_plot = ok
	   end if
	else
	   if( old_itv .ne. 2 ) then
C ***
C ***	      switch paper -> make new screen plot
C ***
              call plote( 3 )	! close paper plot
	      call phard	! print it
              call plot_noerase ! set no erase flag in mongo stuff
	      call ploto( xsize, ysize, .true., .false. )    ! back to screen
	      new_plot = no_refresh
	      new_plot = ok	! pgplot
	   else
C ***
C ***	      no switch, new screen plot
C ***
	      call ploto( xsize, ysize, .true., .false. )    ! back to screen
	      new_plot = ok
	   end if
	end if

	old_itv = itv


	end
                        
