	subroutine resize_add_data( x, y, npkt, sty, icol )
C ***
C ******************************************************************************
C ***
C ***	Routine to add data arrays for resize.
C ***
C ***	This adds just one curve to the curves to be plotted and increments
C ***	ncurv (in common).
C ***
C ***	x(npkt)		r >	x values
C ***	y(npkt)		r >	y values
C ***	npkt		i >	number of points in curve
C ***	sty		r >	line style for curve
C ***	icol		i >	color index for curve
C ***
C ***
C ***	13-JUN-1996 12:21:03.02		ML / Bamberg.  Created.
C ***    3-OCT-2001 12:33:23.00		ML / Stegaurach.  Allocatable memory.
C ***
C ******************************************************************************
C ***
	implicit none


C ***
C ***   Arguments
C ***
        integer npkt
	real x(npkt), y(npkt), sty
	integer icol

C ***
C ***	Gobal variables
C ***
	include 'RESIZE.INC'

C ***
C ***	Local variables
C ***
#if defined( CRAY_PTR ) || defined( REF_PTR )
	integer chunk
	parameter( chunk = 20 )	! chunks (no.curves) in which to allocate memory

C ***
C ***	Functions
C ***
	integer lib$get_vm, lib$free_vm
	external lib$get_vm, lib$free_vm
#endif

	logical first /.true./
	integer status, i

! pointer size:  AXP/VAX:
#ifndef PTRSIZ
#ifdef __ALPHA
#define PTRSIZ 8
#else
#define PTRSIZ 4
#endif
#endif

#if defined( CRAY_PTR )
	integer nnc(*), ncc(*)
	real nsc(*)
	integer*PTRSIZ nxc(*), nyc(*)

	pointer( npp_xdata, nxc )
	pointer( npp_ydata, nyc )
	pointer( np_ndata, nnc )
	pointer( np_style, nsc )
	pointer( np_color, ncc )
#elif defined( REF_PTR )
	integer*PTRSIZ npp_xdata
	integer*PTRSIZ npp_ydata
	integer*PTRSIZ np_ndata
	integer*PTRSIZ np_style
	integer*PTRSIZ np_color

#endif
	

	save first


C ***
C ***	Code
C ***
	if( first ) then
           first = .false.
	   ncurv = 0
#if defined( CRAY_PTR ) || defined( REF_PTR )
           ncu = 0
#endif
	end if

	ncurv = ncurv + 1
#if defined( CRAY_PTR ) || defined( REF_PTR )
	if( ncu .lt. ncurv ) then

C ***
C ***      Get new memory
C ***
           i = ncu + chunk
	   status = lib$get_vm( PTRSIZ*i, npp_xdata )
	   if( status .ne. 1 ) call lib$signal( %val(status) )
	   status = lib$get_vm( PTRSIZ*i, npp_ydata )
	   if( status .ne. 1 ) call lib$signal( %val(status) )
	   status = lib$get_vm( 4*i, np_ndata )
	   if( status .ne. 1 ) call lib$signal( %val(status) )
	   status = lib$get_vm( 4*i, np_style )
	   if( status .ne. 1 ) call lib$signal( %val(status) )
	   status = lib$get_vm( 4*i, np_color )
	   if( status .ne. 1 ) call lib$signal( %val(status) )

C ***
C ***      copy the old data to the new location
C ***
#if defined( CRAY_PTR )
	   do i = 1, ncurv-1
              nxc(i) = xc(i) 
	      nyc(i) = yc(i) 
	      nnc(i) = nc(i) 
	      ncc(i) = cc(i) 
	      nsc(i) = sc(i)
	   end do
#elif defined( REF_PTR )
	   call iarrcp( %val(pp_xdata), %val(npp_xdata), 1, ncurv-1 )
	   call iarrcp( %val(pp_ydata), %val(npp_ydata), 1, ncurv-1 )
	   call iarrcp( %val(p_ndata), %val(np_ndata), 1, ncurv-1 )
	   call iarrcp( %val(p_color), %val(np_color), 1, ncurv-1 )
	   call arrcp( %val(p_style), %val(np_style), 1, ncurv-1 )
#endif

           if( ncu .gt. 0 ) then
C ***
C ***         Free the old memory
C ***
	      status = lib$free_vm( 8*ncu, pp_xdata )
	      if( status .ne. 1 ) call lib$signal( %val(status) )
	      status = lib$free_vm( 8*ncu, pp_ydata )
	      if( status .ne. 1 ) call lib$signal( %val(status) )
	      status = lib$free_vm( 4*ncu, p_ndata )
	      if( status .ne. 1 ) call lib$signal( %val(status) )
	      status = lib$free_vm( 4*ncu, p_style )
	      if( status .ne. 1 ) call lib$signal( %val(status) )
	      status = lib$free_vm( 4*ncu, p_color )
	      if( status .ne. 1 ) call lib$signal( %val(status) )
           end if

C ***
C ***      Fix pointers
C ***      
	   pp_xdata = npp_xdata
           pp_ydata = npp_ydata
           p_ndata =  np_ndata 
           p_style =  np_style 
           p_color =  np_color 


           ncu = ncu + chunk	! remember what we've got
	end if

#if defined( CRAY_PTR )
        xc(ncurv) = %loc(x(1))
	yc(ncurv) = %loc(y(1))
	nc(ncurv) = npkt
	cc(ncurv) = icol
	sc(ncurv) = sty
#elif defined( REF_PTR )
	call iderefass( %ref(%loc(x(1))), %val(pp_xdata), ncurv )
	call iderefass( %ref(%loc(y(1))), %val(pp_ydata), ncurv )
	call iderefass( npkt, %val(p_ndata), ncurv )
	call iderefass( icol, %val(p_color), ncurv )
	call derefass( sty, %val(p_style), ncurv )
#endif

#else
        if( ncurv .gt. max_curv ) then
           write( stdout, * ) 'RESIZE>>  Can''t add another curve.',
     >	                      '  Not enough memory.'
           call exit
        end if
        if( npkt .gt. max_pkt ) then
	   write( stdout, * ) 'RESIZE>> Too many points in curve',
     >                        'to add.  Truncating it.'
        end if
       
	do i = 1, min( npkt, max_pkt )
	   xc(i,ncurv) = x(i)
	   yc(i,ncurv) = y(i)
	end do
	nc(ncurv) = min( npkt, max_pkt )
	cc(ncurv) = icol
	sc(ncurv) = sty
	
#endif

	end
